#!/usr/bin/env python
"""

@author: Alex Drlica-Wagner <kadrlica@fnal.gov>
"""
import os
import argparse

import numpy as np
import numpy.lib.recfunctions as recfuncs
import pyfits

from ugali.utils.healpix import pix2ang
from ugali.utils.projector import cel2gal
from ugali.utils.logger import logger

class Parser(argparse.ArgumentParser):
    def __init__(self,*args,**kwargs):
        kwargs.setdefault('formatter_class',
                          argparse.ArgumentDefaultsHelpFormatter)

        super(Parser,self).__init__(*args,**kwargs)


    def add_coords(self,required=False,radius=False,targets=False):
        group = self.add_argument_group(title="Target Coordinates")
        exclusive = group.add_mutually_exclusive_group(required=required)
        if targets:
            exclusive.add_argument('-t','--targets',default=None,metavar='targets.txt',
                                   help="List of target coordinates")
        exclusive.add_argument('--gal',nargs=2,default=None,metavar=('glon','glat'),
                            type=float,help="Location in Galactic coordinates.")
        exclusive.add_argument('--cel',nargs=2,default=None,metavar=('ra','dec'),
                            type=float,help="Location in celestial coordinates.")
        exclusive.add_argument('--hpx',nargs=2,default=None,metavar=('nside','pix'),
                            type=int,help="Location in HEALPix coordinates.")
        if radius:
            group.add_argument('--radius',default=0,type=float,
                               help="Radius surrounding specified coordinates")

    def add_name(self,**kwargs):
        self.add_argument('-n','--name',
                          help = "Target name",**kwargs)

    def add_verbose(self,**kwargs):
        self.add_argument('-v','--verbose',action='store_true',
                          help='Output verbosity.',**kwargs)

    def add_debug(self,**kwargs):
        self.add_argument('-d','--debug',action='store_true',
                          help="Setup, but do not run",**kwargs)

    def add_queue(self,**kwargs):
        self.add_argument('-q','--queue',
                          help="Batch queue for execution.",**kwargs)
                          

    def add_config(self,**kwargs):
        self.add_argument('config',metavar='config.yaml',
                          help='Configuration file (yaml or python dict).',**kwargs)


    def add_force(self,**kwargs):
        self.add_argument('-f','--force',action='store_true',
                          help='Force the overwrite of files',**kwargs)

    def add_seed(self,**kwargs):
        self.add_argument('--seed',default=None,type=int,
                          help='Random seed.',**kwargs)

    def add_version(self,**kwargs):
        from ugali import __version__
        self.add_argument('-V','--version',action='version',
                          version='ugali '+__version__,
                          help='Print version.',**kwargs)

    def add_run(self,**kwargs):
        self.add_argument('-r','--run', default=[], action='append',
                          help="Analysis component(s) to run.", **kwargs)

    def _parse_verbose(self,opts):
        if vars(opts).get('verbose'): 
            logger.setLevel(logger.DEBUG)

    def _parse_coords(self,opts):
        # The coordinates are mutually exclusive, so
        # shouldn't have to worry about over-writing them.
        if 'coords' in vars(opts): return
        radius = vars(opts).get('radius',0)
        gal = None
        if vars(opts).get('gal') is not None: 
            gal = opts.gal
        elif vars(opts).get('cel') is not None: 
            gal = cel2gal(*opts.cel)
        elif vars(opts).get('hpx') is not None: 
            gal = pix2ang(*opts.hpx)

        if gal is not None:
            opts.coords = [(gal[0],gal[1],radius)]
            opts.names = [vars(opts).get('name','')]
        else:
            opts.coords = None
            opts.names = None

        if vars(opts).get('targets') is not None:
            opts.names,opts.coords = self.parse_targets(opts.targets)
            if vars(opts).get('radius') is not None:
                opts.coords[:,2] = vars(opts).get('radius')
            
    @staticmethod
    def parse_targets(filename):
        """
        Load a text file with target coordinates. Returns
        an array of target locations in Galactic coordinates.
        File description:
        [NAME] [LON] [LAT] [RADIUS] [COORD]
        
        The values of LON and LAT will depend on COORD:
        COORD = [GAL  | CEL | HPX  ],
        LON   = [GLON | RA  | NSIDE]
        LAT   = [GLAT | DEC | PIX  ]

        """
        base,ext = os.path.splitext(filename)
        if ext == '.fits':
            f = pyfits.open(filename)
            data = f[1].data.view(np.recarray)
            data = recfuncs.append_fields(data,'RADIUS',np.zeros(len(data)),usemask=False)
            return data['NAME'],data[['GLON','GLAT','RADIUS']]
        elif (ext=='.txt') or (ext=='.dat'):
            data = np.loadtxt(filename,unpack=True,usecols=range(5),dtype=object)
            # Deal with one-line input files
            if data.ndim == 1: data = np.array([data]).T
            names = data[0]
            out   = data[1:4].astype(float)
            lon,lat,radius = out
             
            coord = np.array([s.lower() for s in data[4]])
            gal = (coord=='gal')
            cel = (coord=='cel')
            hpx = (coord=='hpx')
             
            if cel.any():
                glon,glat = cel2gal(lon[cel],lat[cel])
                out[0][cel] = glon
                out[1][cel] = glat
            if hpx.any():
                glon,glat = pix2ang(lat[hpx],lon[hpx])
                out[0][hpx] = glon
                out[1][hpx] = glat
             
            return names,out.T

    #def _parse_local(self,opts):
    #    if vars(opts).get('local') is not None: return
    #    opts.local = (vars(opts).get('queue')=='local')

    def parse_args(self,*args,**kwargs):
        opts = super(Parser,self).parse_args(*args,**kwargs)
        self._parse_verbose(opts)
        self._parse_coords(opts)
        #self._parse_local(opts)
        return opts

if __name__ == "__main__":
    description = "Argument parser test."
    parser = Parser(description=description)
    parser.add_debug()
    parser.add_verbose()
    parser.add_version()
    parser.add_coords()
    opts = parser.parse_args()
    print opts



#!/usr/bin/env python
"""
Class to create and run an individual likelihood analysis.

Classes:
    Scan -- ADW: Doesn't do anything
    GridSearch -- ADW: Should be renamed to Scan

"""

import os
import sys
from collections import OrderedDict as odict

import numpy
import numpy as np
import pyfits
import healpy

import ugali.utils.skymap
import ugali.analysis.loglike
from ugali.analysis.loglike import LogLikelihood, createSource, createObservation
from ugali.analysis.source import Source
from ugali.utils.parabola import Parabola

from ugali.utils.config import Config
from ugali.utils.logger import logger
from ugali.utils.healpix import superpixel, subpixel, pix2ang, ang2pix

############################################################

class Scan(object):
    """
    The base of a likelihood analysis scan.

    FIXME: This really does nothing now and should be absorbed into
    GridSearch (confusing now).
    """
    def __init__(self, config, coords):
        self.config = Config(config)
        # Should only be one coordinate
        if len(coords) != 1: raise Exception('Must specify one coordinate.')
        self.lon,self.lat,radius = coords[0]
        self._setup()
        

    def _setup(self):
        self.observation = createObservation(self.config,self.lon,self.lat)
        self.source = createSource(self.config,'scan',lon=self.lon,lat=self.lat)
        loglike=LogLikelihood(self.config,self.observation,self.source)
        self.grid = GridSearch(self.config,loglike)

    @property
    def loglike(self):
        return self.grid.loglike

    def run(self, coords=None, debug=False):
        """
        Run the likelihood grid search
        """
        #self.grid.precompute()
        self.grid.search(coords=coords)
        return self.grid
        
    def write(self, outfile):
        self.grid.write(outfile)


def createGridSearch(config,lon,lat):
    config = Config(config)
    obs = createObservation(config,lon,lat)
    src = createSource(config,'scan',lon=lon,lat=lat)
    loglike=LogLikelihood(config,obs,src)
    return GridSearch(config,loglike)
    

############################################################

class GridSearch:

    #def __init__(self, config, roi, mask, catalog, isochrone, kernel):
    #def __init__(self, config, observation, source):
    def __init__(self, config, loglike): # What it should be...
        """
        Object to efficiently search over a grid of ROI positions.
        """

        self.config = Config(config)
        self.loglike = loglike
        self.roi  = self.loglike.roi
        self.mask = self.loglike.mask

        #logger.info("Creating log-likelihood...")
        #self.loglike=LogLikelihood(config,roi,mask,catalog,isochrone,kernel)
        #self.loglike=LogLikelihood(config,observation,source)

        logger.info(str(self.loglike))

        self.stellar_mass_conversion = self.loglike.source.stellar_mass()
        self.distance_modulus_array = np.asarray(self.config['scan']['distance_modulus_array'])

    def precompute(self, distance_modulus_array=None):
        """
        Precompute u_background and u_color for each star in catalog.
        Precompute observable fraction in each ROI pixel.
        # Precompute still operates over the full ROI, not just the likelihood region
        """
        if distance_modulus_array is not None:
            self.distance_modulus_array = distance_modulus_array
        else:
            self.distance_modulus_array = sel

        # Observable fraction for each pixel
        self.u_color_array = [[]] * len(self.distance_modulus_array)
        self.observable_fraction_sparse_array = [[]] * len(self.distance_modulus_array)

        logger.info('Looping over distance moduli in precompute ...')
        for ii, distance_modulus in enumerate(self.distance_modulus_array):
            logger.info('  (%i/%i) Distance Modulus = %.2f ...'%(ii+1, len(self.distance_modulus_array), distance_modulus))

            self.u_color_array[ii] = False
            if self.config['scan']['color_lut_infile'] is not None:
                logger.info('  Precomputing signal color from %s'%(self.config['scan']['color_lut_infile']))
                self.u_color_array[ii] = ugali.analysis.color_lut.readColorLUT(self.config['scan']['color_lut_infile'],
                                                                               distance_modulus,
                                                                               self.loglike.catalog.mag_1,
                                                                               self.loglike.catalog.mag_2,
                                                                               self.loglike.catalog.mag_err_1,
                                                                               self.loglike.catalog.mag_err_2)
            if not numpy.any(self.u_color_array[ii]):
                logger.info('  Precomputing signal color on the fly...')
                self.u_color_array[ii] = self.loglike.calc_signal_color(distance_modulus) 
            
            # Calculate over all pixels in ROI
            self.observable_fraction_sparse_array[ii] = self.loglike.calc_observable_fraction(distance_modulus)

        self.u_color_array = numpy.array(self.u_color_array)

                
    def search(self, coords=None, distance_modulus=None, tolerance=1.e-2):
        """
        Organize a grid search over ROI target pixels and distance moduli in distance_modulus_array
        coords: (lon,lat)
        distance_modulus: scalar
        """
        nmoduli = len(self.distance_modulus_array)
        npixels    = len(self.roi.pixels_target)
        self.log_likelihood_sparse_array       = numpy.zeros([nmoduli, npixels])
        self.richness_sparse_array             = numpy.zeros([nmoduli, npixels])
        self.richness_lower_sparse_array       = numpy.zeros([nmoduli, npixels])
        self.richness_upper_sparse_array       = numpy.zeros([nmoduli, npixels])
        self.richness_upper_limit_sparse_array = numpy.zeros([nmoduli, npixels])
        self.stellar_mass_sparse_array         = numpy.zeros([nmoduli, npixels])
        self.fraction_observable_sparse_array  = numpy.zeros([nmoduli, npixels])

        # Specific pixel/distance_modulus
        coord_idx, distance_modulus_idx = None, None
        if coords is not None:
            # Match to nearest grid coordinate index
            coord_idx = self.roi.indexTarget(coords[0],coords[1])
        if distance_modulus is not None:
            # Match to nearest distance modulus index
            distance_modulus_idx=np.fabs(self.distance_modulus_array-distance_modulus).argmin()

        lon, lat = self.roi.pixels_target.lon, self.roi.pixels_target.lat
            
        logger.info('Looping over distance moduli in grid search ...')
        for ii, distance_modulus in enumerate(self.distance_modulus_array):

            # Specific pixel
            if distance_modulus_idx is not None:
                if ii != distance_modulus_idx: continue

            logger.info('  (%-2i/%i) Distance Modulus=%.1f ...'%(ii+1,nmoduli,distance_modulus))

            # Set distance_modulus once to save time
            self.loglike.set_params(distance_modulus=distance_modulus)

            for jj in range(0, npixels):
                # Specific pixel
                if coord_idx is not None:
                    if jj != coord_idx: continue

                # Set kernel location
                self.loglike.set_params(lon=lon[jj],lat=lat[jj])
                # Doesn't re-sync distance_modulus each time
                self.loglike.sync_params()
                                         
                args = (jj+1, npixels, self.loglike.source.lon, self.loglike.source.lat)
                message = '    (%-3i/%i) Candidate at (%.2f, %.2f) ... '%(args)

                self.log_likelihood_sparse_array[ii][jj], self.richness_sparse_array[ii][jj], parabola = self.loglike.fit_richness()
                self.stellar_mass_sparse_array[ii][jj] = self.stellar_mass_conversion * self.richness_sparse_array[ii][jj]
                self.fraction_observable_sparse_array[ii][jj] = self.loglike.f
                if self.config['scan']['full_pdf']:
                    #n_pdf_points = 100
                    #richness_range = parabola.profileUpperLimit(delta=25.) - self.richness_sparse_array[ii][jj]
                    #richness = numpy.linspace(max(0., self.richness_sparse_array[ii][jj] - richness_range),
                    #                          self.richness_sparse_array[ii][jj] + richness_range,
                    #                          n_pdf_points)
                    #if richness[0] > 0.:
                    #    richness = numpy.insert(richness, 0, 0.)
                    #    n_pdf_points += 1
                    # 
                    #log_likelihood = numpy.zeros(n_pdf_points)
                    #for kk in range(0, n_pdf_points):
                    #    log_likelihood[kk] = self.loglike.value(richness=richness[kk])
                    #parabola = ugali.utils.parabola.Parabola(richness, 2.*log_likelihood)
                    #self.richness_lower_sparse_array[ii][jj], self.richness_upper_sparse_array[ii][jj] = parabola.confidenceInterval(0.6827)
                    self.richness_lower_sparse_array[ii][jj], self.richness_upper_sparse_array[ii][jj] = self.loglike.richness_interval(0.6827)
                    
                    self.richness_upper_limit_sparse_array[ii][jj] = parabola.bayesianUpperLimit(0.95)

                    args = (
                        2. * self.log_likelihood_sparse_array[ii][jj],
                        self.stellar_mass_conversion*self.richness_sparse_array[ii][jj],
                        self.stellar_mass_conversion*self.richness_lower_sparse_array[ii][jj],
                        self.stellar_mass_conversion*self.richness_upper_sparse_array[ii][jj],
                        self.stellar_mass_conversion*self.richness_upper_limit_sparse_array[ii][jj]
                    )
                    message += 'TS=%.1f, Stellar Mass=%.1f (%.1f -- %.1f @ 0.68 CL, < %.1f @ 0.95 CL)'%(args)
                else:
                    args = (
                        2. * self.log_likelihood_sparse_array[ii][jj], 
                        self.stellar_mass_conversion * self.richness_sparse_array[ii][jj],
                        self.fraction_observable_sparse_array[ii][jj]
                    )
                    message += 'TS=%.1f, Stellar Mass=%.1f, Fraction=%.2g'%(args)
                logger.debug( message )
                
                #if coords is not None and distance_modulus is not None:
                #    results = [self.richness_sparse_array[ii][jj],
                #               self.log_likelihood_sparse_array[ii][jj],
                #               self.richness_lower_sparse_array[ii][jj],
                #               self.richness_upper_sparse_array[ii][jj],
                #               self.richness_upper_limit_sparse_array[ii][jj],
                #               richness, log_likelihood, self.loglike.p, self.loglike.f]
                #    return results

            jj_max = self.log_likelihood_sparse_array[ii].argmax()
            args = (
                jj_max+1, npixels, lon[jj_max], lat[jj_max],
                2. * self.log_likelihood_sparse_array[ii][jj_max], 
                self.stellar_mass_conversion * self.richness_sparse_array[ii][jj_max]
            )
            message = '  (%-3i/%i) Maximum at (%.2f, %.2f) ... TS=%.1f, Stellar Mass=%.1f'%(args)
            logger.info( message )
            #outfile = 'srcmdl_%i.yaml'%ii
            #print "Writing %s..."%outfile
            #self.loglike.source.write(outfile)
            
    def mle(self):
        a = self.log_likelihood_sparse_array
        j,k = np.unravel_index(a.argmax(),a.shape)
        mle = odict()
        mle['richness'] = self.richness_sparse_array[j][k]
        mle['lon'] = self.roi.pixels_target.lon[k]
        mle['lat'] = self.roi.pixels_target.lat[k]
        mle['distance_modulus'] = self.distance_modulus_array[j]
        mle['extension'] = float(self.loglike.source.extension)
        mle['ellipticity'] = float(self.loglike.source.ellipticity)
        mle['position_angle'] = float(self.loglike.source.position_angle)
        # ADW: FIXME!
        try: 
            mle['age'] = np.mean(self.loglike.source.age)
            mle['metallicity'] = np.mean(self.loglike.source.metallicity)
        except AttributeError:
            mle['age'] = np.nan
            mle['metallicity'] = np.nan
            
        return mle

    def err(self):
        """
        A few rough approximations of the fit uncertainty. These
        values shouldn't be trusted for anything real (use MCMC instead).
        """
        # Initiallize error to nan
        err = odict(self.mle())
        err.update([(k,np.nan*np.ones(2)) for k in err.keys()])

        # Find the maximum likelihood
        a = self.log_likelihood_sparse_array
        j,k = np.unravel_index(a.argmax(),a.shape)

        self.loglike.set_params(distance_modulus=self.distance_modulus_array[j],
                                lon=self.roi.pixels_target.lon[k],
                                lat=self.roi.pixels_target.lat[k])
        self.loglike.sync_params()

        # Find the error in richness, starting at maximum
        lo,hi = np.array(self.loglike.richness_interval())
        err['richness'] = np.array([lo,hi])

        # ADW: This is a rough estimate of the distance uncertainty 
        # hacked to keep the maximum distance modulus on a grid index

        # This is a hack to get the confidence interval to play nice...
        # Require at least three points.
        if (a[:,k]>0).sum() >= 3:
            parabola = Parabola(np.insert(self.distance_modulus_array,0,0.), 
                                np.insert(a[:,k],0,0.) )
            lo,hi = np.array(parabola.confidenceInterval())
            err['distance_modulus'] = self.distance_modulus_array[j] + (hi-lo)/2.*np.array([-1.,1.])

        # ADW: Could estimate lon and lat from the grid.
        # This is just concept right now...
        if (a[j,:]>0).sum() >= 10:
            delta_ts = 2*(a[j,k] - a[j,:])
            pix = np.where(a[j,:][delta_ts < 2.71])[0]
            lons = self.roi.pixels_target.lon[pix]
            lats = self.roi.pixels_target.lat[pix]
            err['lon'] = np.array([ np.min(lons),np.max(lons)])
            err['lat'] = np.array([ np.min(lats),np.max(lats)])

        return err

    def write(self, outfile):
        """
        Save the likelihood fitting results as a sparse HEALPix map.
        """
        # Full data output (too large for survey)
        if self.config['scan']['full_pdf']:
            data_dict = {'LOG_LIKELIHOOD': self.log_likelihood_sparse_array.transpose(),
                         'RICHNESS':       self.richness_sparse_array.transpose(),
                         'RICHNESS_LOWER': self.richness_lower_sparse_array.transpose(),
                         'RICHNESS_UPPER': self.richness_upper_sparse_array.transpose(),
                         'RICHNESS_LIMIT': self.richness_upper_limit_sparse_array.transpose(),
                         #'STELLAR_MASS': self.stellar_mass_sparse_array.transpose(),
                         'FRACTION_OBSERVABLE': self.fraction_observable_sparse_array.transpose()}
        else:
            data_dict = {'LOG_LIKELIHOOD': self.log_likelihood_sparse_array.transpose(),
                         'RICHNESS': self.richness_sparse_array.transpose(),
                         'FRACTION_OBSERVABLE': self.fraction_observable_sparse_array.transpose()}

        # Stellar Mass can be calculated from STELLAR * RICHNESS
        header_dict = {
            'STELLAR' : round(self.stellar_mass_conversion,8),
            'LKDNSIDE': self.config['coords']['nside_likelihood'],
            'LKDPIX'  : ang2pix(self.config['coords']['nside_likelihood'],self.roi.lon,self.roi.lat),
            'NROI'    : self.roi.inROI(self.loglike.catalog_roi.lon,self.loglike.catalog_roi.lat).sum(), 
            'NANNULUS': self.roi.inAnnulus(self.loglike.catalog_roi.lon,self.loglike.catalog_roi.lat).sum(), 
            'NINSIDE' : self.roi.inInterior(self.loglike.catalog_roi.lon,self.loglike.catalog_roi.lat).sum(), 
            'NTARGET' : self.roi.inTarget(self.loglike.catalog_roi.lon,self.loglike.catalog_roi.lat).sum(), 
        }

        # In case there is only a single distance modulus
        if len(self.distance_modulus_array) == 1:
            for key in data_dict:
                data_dict[key] = data_dict[key].flatten()

        ugali.utils.skymap.writeSparseHealpixMap(self.roi.pixels_target,
                                                 data_dict,
                                                 self.config['coords']['nside_pixel'],
                                                 outfile,
                                                 distance_modulus_array=self.distance_modulus_array,
                                                 coordsys='NULL', ordering='NULL',
                                                 header_dict=header_dict)

############################################################
    
if __name__ == "__main__":
    import ugali.utils.parser
    description = "Script for executing the likelihood scan."
    parser = ugali.utils.parser.Parser(description=description)
    parser.add_config()
    parser.add_argument('outfile',metavar='outfile.fits',help='Output fits file.')
    parser.add_debug()
    parser.add_verbose()
    parser.add_coords(required=True,radius=False)
    opts = parser.parse_args()

    if len(opts.coords) != 1: 
        raise Exception('Must specify exactly one coordinate.')
    lon,lat,radius = opts.coords[0]

    grid = createGridSearch(opts.config,lon,lat)
    if not opts.debug:
        result = grid.search()
        grid.write(opts.outfile)


    ##print opts.coords
    #scan = Scan(opts.config,opts.coords)
    #if not opts.debug:
    #    result = scan.run()
    #    scan.write(opts.outfile)



#!/usr/bin/env python
"""
For pixelizing catalogs and masks.
"""
import os
from os.path import join

import pyfits
import numpy
import numpy.lib.recfunctions as recfuncs
import collections
import healpy

import ugali.utils.binning
import ugali.utils.skymap
from ugali.utils.projector import cel2gal, gal2cel
from ugali.utils.healpix import ang2pix, pix2ang, superpixel
from ugali.utils.shell import mkdir
from ugali.utils.logger import logger
from ugali.utils.config import Config

def pixelizeCatalog(infiles, config, force=False):
    """
    Break catalog up into a set of healpix files.
    """
    nside_catalog = config['coords']['nside_catalog']
    nside_pixel = config['coords']['nside_pixel']
    outdir = mkdir(config['catalog']['dirname'])
    filenames = config.getFilenames()
    
    for ii,infile in enumerate(infiles):
        logger.info('(%i/%i) %s'%(ii+1, len(infiles), infile))
        f = pyfits.open(infile)
        data = f[1].data
        header = f[1].header
        logger.info("%i objects found"%len(data))
        if not len(data): continue
        glon,glat = cel2gal(data['RA'],data['DEC'])
        catalog_pix = ang2pix(nside_catalog,glon,glat,coord='GAL')
        pixel_pix = ang2pix(nside_pixel,glon,glat,coord='GAL')
        names = [n.upper() for n in data.columns.names]
        ra_idx = names.index('RA'); dec_idx = names.index('DEC')
        idx = ra_idx if ra_idx > dec_idx else dec_idx
        catalog_pix_name = 'PIX%i'%nside_catalog
        pixel_pix_name = 'PIX%i'%nside_pixel

        coldefs = pyfits.ColDefs(
            [pyfits.Column(name='GLON',format='1D',array=glon),
             pyfits.Column(name='GLAT',format='1D',array=glat),
             pyfits.Column(name=catalog_pix_name,format='1J',array=catalog_pix),
             pyfits.Column(name=pixel_pix_name  ,format='1J',array=pixel_pix)]
        )
        hdu = pyfits.new_table(data.columns[:idx+1]+coldefs+data.columns[idx+1:])
        table = hdu.data

        for pix in numpy.unique(catalog_pix):
            logger.debug("Processing pixel %s"%pix)
            outfile = filenames.data['catalog'][pix]
            if not os.path.exists(outfile):
                logger.debug("Creating %s"%outfile)
                names = [n.upper() for n in table.columns.names]
                formats = table.columns.formats
                columns = [pyfits.Column(n,f) for n,f in zip(names,formats)]
                out = pyfits.HDUList([pyfits.PrimaryHDU(),pyfits.new_table(columns)])
                out[1].header['NSIDE'] = nside_catalog
                out[1].header['PIX'] = pix
                out.writeto(outfile)
            hdulist = pyfits.open(outfile,mode='update')
            t1 = hdulist[1].data
            # Could we speed up with sorting and indexing?
            t2 = table[ table[catalog_pix_name] == pix ]
            nrows1 = t1.shape[0]
            nrows2 = t2.shape[0]
            nrows = nrows1 + nrows2
            out = pyfits.new_table(t1.columns, nrows=nrows)
            for name in t1.columns.names:
                out.data.field(name)[nrows1:]=t2.field(name)
            hdulist[1] = out
            logger.debug("Writing %s"%outfile)
            hdulist.flush()
            hdulist.close()

def pixelizeDensity(config, nside=None, force=False):
    if nside is None: 
        nside = config['coords']['nside_likelihood']
    filenames = config.getFilenames()
    infiles = filenames[~filenames['catalog'].mask]

    for ii,f in enumerate(infiles.data):
        infile = f['catalog']
        pix = f['pix']
        logger.info('(%i/%i) %s'%(ii+1, len(infiles), infile))

        outfile = config['data']['density']%pix
        if os.path.exists(outfile) and not force: 
            logger.info("Found %s; skipping..."%outfile)
            continue
            
        outdir = mkdir(os.path.dirname(outfile))
        pixels, density = stellarDensity(infile,nside)
        data_dict = dict( DENSITY=density )
        logger.info("Writing %s..."%outfile)
        ugali.utils.skymap.writeSparseHealpixMap(pixels,data_dict,nside,outfile)

def stellarDensity(infile, nside=2**8): 
    area = healpy.nside2pixarea(nside,degrees=True)
    f = pyfits.open(infile)
    data = f[1].data
    logger.debug("Reading %s"%infile)
    
    glon,glat = data['GLON'],data['GLAT']
    pix = ang2pix(nside,glon,glat,coord='GAL')
    counts = collections.Counter(pix)
    pixels, number = numpy.array(sorted(counts.items())).T
    density = number/area
    f.close()

    return pixels, density

if __name__ == "__main__":
    from optparse import OptionParser
    usage = "Usage: %prog  [options] input"
    description = "python script"
    parser = OptionParser(usage=usage,description=description)
    (opts, args) = parser.parse_args()



"""
Class to construct parabolas from 3 points.

ADW: Need to move all of the plotting stuff
"""

import numpy
import scipy.stats
import scipy.interpolate

############################################################

class Parabola:

    def __init__(self, x, y):
        """
        INPUTS
        x = variable of interest
        y = 2 * log(likelihood)
        """

        # Sort the input
        argsort = numpy.argsort(x)
        self.x = numpy.array(x)[argsort]
        self.y = numpy.array(y)[argsort]

        index = numpy.argmax(self.y)
        if index == 0:
            index_0 = 0
            index_1 = 1
            index_2 = 2
        elif index == len(self.y) - 1:
            index_0 = len(self.y) - 3
            index_1 = len(self.y) - 2
            index_2 = len(self.y) - 1
        else:
            index_0 = index - 1
            index_1 = index
            index_2 = index + 1

        x_0 = self.x[index_0]
        x_1 = self.x[index_1]
        x_2 = self.x[index_2]
        y_0 = self.y[index_0]
        y_1 = self.y[index_1]
        y_2 = self.y[index_2]

        # Invert matrix
        a = numpy.matrix([[x_0**2, x_0, 1.],
                          [x_1**2, x_1, 1.],
                          [x_2**2, x_2, 1.]])
        a_inverse = numpy.linalg.inv(a)
        b = numpy.array([y_0, y_1, y_2])
        p = numpy.dot(numpy.array(a_inverse), b)

        self.p_2 = p[0]
        self.p_1 = p[1]
        self.p_0 = p[2]
         
        # Vertex
        self.vertex_x = -self.p_1 / (2. * self.p_2)
        self.vertex_y = self.p_0 - (self.p_1**2 / (4. * self.p_2))

    def __eq__(self,other):
        return numpy.allclose([self.p_0,self.p_1,self.p_2],[other.p_0,other.p_1,other.p_2])

    def __ne__(self,other):
        return not self.__eq__(other)
        
    def __repr__(self):
        return "y = %.2g * x**2 + %.2g * x + %.2g"%(self.p_2, self.p_1, self.p_0)

    def __str__(self):
        return self.__repr__()

    def __call__(self, x):
        """
        Evaluate the parabola.
        """
        return (self.p_2 * x**2) + (self.p_1 * x) + self.p_0

    def densify(self, factor=10):
        """
        Increase the density of points along the parabolic curve.
        """
        x = []
        y = []
        for ii in range(0, len(self.x) - 2):
            p = Parabola(self.x[ii: ii + 3], self.y[ii: ii + 3])
            x.append(numpy.linspace(self.x[ii], self.x[ii + 1], factor)[0: -1])
            y.append(p(x[-1]))

        p = Parabola(self.x[len(self.x) - 3:], self.y[len(self.y) - 3:])
        x.append(numpy.linspace(self.x[-2], self.x[-1], factor)[0: -1])
        y.append(p(x[-1]))

        x.append([self.x[-1]])
        y.append([self.y[-1]])

        #f = scipy.interpolate.interp1d(numpy.concatenate(x), numpy.concatenate(y))
        #x = numpy.linspace(self.x[0], self.x[-1], len(x) * factor)   
        #return x, f(x)
        
        return numpy.concatenate(x), numpy.concatenate(y)

    def profileUpperLimit(self, delta = 2.71):
        """
        Compute one-sided upperlimit via profile method.
        """
        a = self.p_2
        b = self.p_1
        if self.vertex_x < 0:
            c = self.p_0 + delta
        else:
            c = self.p_0 - self.vertex_y + delta

        if b**2 - 4. * a * c < 0.:
            print 'WARNING'

            print a, b, c
            
            #pylab.figure()
            #pylab.scatter(self.x, self.y)
            #raw_input('WAIT')
            return 0.

        
            
        return max((numpy.sqrt(b**2 - 4. * a * c) - b) / (2. * a), (-1. * numpy.sqrt(b**2 - 4. * a * c) - b) / (2. * a)) 

    #def bayesianUpperLimit3(self, alpha, steps = 1.e5):
    #    """
    #    Compute one-sided upper limit using Bayesian Method of Helene.
    #    """
    #    # Need a check to see whether limit is reliable
    #    pdf = scipy.interpolate.interp1d(self.x, numpy.exp(self.y / 2.)) # Convert from 2 * log(likelihood) to likelihood
    #    x_pdf = numpy.linspace(self.x[0], self.x[-1], steps)
    #    cdf = numpy.cumsum(pdf(x_pdf))
    #    cdf /= cdf[-1]
    #    cdf_reflect = scipy.interpolate.interp1d(cdf, x_pdf)
    #    return cdf_reflect(alpha)
    #    #return self.x[numpy.argmin((cdf - alpha)**2)]

    def bayesianUpperLimit(self, alpha, steps=1.e5, plot=False):
        """
        Compute one-sided upper limit using Bayesian Method of Helene.
        Several methods of increasing numerical stability have been implemented.
        """
        x_dense, y_dense = self.densify()
        y_dense -= numpy.max(y_dense) # Numeric stability
        f = scipy.interpolate.interp1d(x_dense, y_dense, kind='linear')
        x = numpy.linspace(0., numpy.max(x_dense), steps)
        pdf = numpy.exp(f(x) / 2.)
        cut = (pdf / numpy.max(pdf)) > 1.e-10
        x = x[cut]
        pdf = pdf[cut]
        #pdf /= pdf[0]
        #forbidden = numpy.nonzero(pdf < 1.e-10)[0]
        #if len(forbidden) > 0:
        #    index = forbidden[0] # Numeric stability
        #    x = x[0: index]
        #    pdf = pdf[0: index]
        cdf = numpy.cumsum(pdf)
        cdf /= cdf[-1]
        cdf_reflect = scipy.interpolate.interp1d(cdf, x)

        #if plot:            
        #    pylab.figure()
        #    pylab.plot(x, f(x))
        #    pylab.scatter(self.x, self.y, c='red')
        #    
        #    pylab.figure()
        #    pylab.plot(x, pdf)
        #    
        #    pylab.figure()
        #    pylab.plot(cdf, x)
        
        return cdf_reflect(alpha)

    def bayesianUpperLimit2(self, alpha, steps=1.e5, plot=False):
        """
        Compute one-sided upper limit using Bayesian Method of Helene.
        """
        cut = ((self.y / 2.) > -30.) # Numeric stability
        try:
            f = scipy.interpolate.interp1d(self.x[cut], self.y[cut], kind='cubic')
        except:
            f = scipy.interpolate.interp1d(self.x[cut], self.y[cut], kind='linear')
        x = numpy.linspace(0., numpy.max(self.x[cut]), steps)
        y = numpy.exp(f(x) / 2.)
        #forbidden = numpy.nonzero((y / numpy.exp(self.vertex_y / 2.)) < 1.e-10)[0]
        forbidden = numpy.nonzero((y / self.vertex_y) < 1.e-10)[0]
        if len(forbidden) > 0:
            index = forbidden[0] # Numeric stability
            x = x[0: index]
            y = y[0: index]
        cdf = numpy.cumsum(y)
        cdf /= cdf[-1]
        cdf_reflect = scipy.interpolate.interp1d(cdf, x)

        #if plot:
        #    pylab.figure()
        #    pylab.scatter(self.x, self.y)
        # 
        #    pylab.figure()
        #    pylab.plot(x, f(x))
        #    
        #    pylab.figure()
        #    pylab.plot(x, y)
        #    
        #    pylab.figure()
        #    pylab.plot(cdf, x)
        
        return cdf_reflect(alpha)

        """
        if numpy.isnan(result):
            import pylab

            for ii in range(0, len(self.x)):
                print '%.3f %.3f'%(self.x[ii], self.y[ii])
            
            pylab.figure()
            pylab.scatter(self.x, self.y)
            pylab.figure()
            pylab.scatter(cdf, x)
            raw_input('WAIT')
        
        return result
        """

    def confidenceInterval(self, alpha=0.6827, steps=1.e5, plot=False):
        """
        Compute two-sided confidence interval by taking x-values corresponding to the largest PDF-values first.
        """
        x_dense, y_dense = self.densify()
        y_dense -= numpy.max(y_dense) # Numeric stability
        f = scipy.interpolate.interp1d(x_dense, y_dense, kind='linear')
        x = numpy.linspace(0., numpy.max(x_dense), steps)
        # ADW: Why does this start at 0, which often outside the input range?
        # Wouldn't starting at xmin be better:
        #x = numpy.linspace(numpy.min(x_dense), numpy.max(x_dense), steps)
        pdf = numpy.exp(f(x) / 2.)
        cut = (pdf / numpy.max(pdf)) > 1.e-10
        x = x[cut]
        pdf = pdf[cut]

        sorted_pdf_indices = numpy.argsort(pdf)[::-1] # Indices of PDF in descending value
        cdf = numpy.cumsum(pdf[sorted_pdf_indices])
        cdf /= cdf[-1]
        sorted_pdf_index_max = numpy.argmin((cdf - alpha)**2)
        x_select = x[sorted_pdf_indices[0: sorted_pdf_index_max]]

        #if plot:
        #    cdf = numpy.cumsum(pdf)
        #    cdf /= cdf[-1]
        #    print cdf[numpy.max(sorted_pdf_indices[0: sorted_pdf_index_max])] \
        #          - cdf[numpy.min(sorted_pdf_indices[0: sorted_pdf_index_max])]
        #    
        #    pylab.figure()
        #    pylab.plot(x, f(x))
        #    pylab.scatter(self.x, self.y, c='red')
        #    
        #    pylab.figure()
        #    pylab.plot(x, pdf)
            
        return numpy.min(x_select), numpy.max(x_select) 

############################################################

def upperLimitsDeltaTS(confidence_level, one_sided=True, degrees_of_freedom=1):
    """

    """
    if not one_sided:
        confidence_level = 0.5*(confidence_level + 1.)
    ts_min = 0 # TS = Test Statistic
    ts_max = 5
    ts_steps = 1000
    x = numpy.linspace(ts_min, ts_max, ts_steps)
    y = (0.5 * scipy.stats.chi2.sf(x, degrees_of_freedom) - (1. - confidence_level))**2
    return x[numpy.argmin(y)]
        
############################################################



from numpy import array, sqrt, zeros, sin, cos, arange, arcsin,\
      arctan2, transpose, concatenate, ndarray, pi, dot, deg2rad,\
      rad2deg

def jprecess(ra, dec, mu_radec=None, parallax=None, rad_vel=None, epoch=None):
   """
    NAME:
         JPRECESS
    PURPOSE:
         Precess astronomical coordinates from B1950 to J2000
    EXPLANATION:
         Calculate the mean place of a star at J2000.0 on the FK5 system from the
         mean place at B1950.0 on the FK4 system.
    
         Use BPRECESS for the reverse direction J2000 ==> B1950
    CALLING SEQUENCE:
         jprecess, ra, dec, ra_2000, dec_2000, [ MU_RADEC = , PARALLAX = 
                  RAD_VEL =, EPOCH =   ]
    
    INPUTS:
         RA,DEC - input B1950 right ascension and declination in *degrees*.
                  Scalar or vector
    
    OUTPUTS:
         RA_2000, DEC_2000 - the corresponding J2000 right ascension and 
                  declination in *degrees*.   Same number of elements as RA,DEC
                  but always double precision. 
    
    OPTIONAL INPUT-OUTPUT KEYWORDS
         MU_RADEC - 2xN element double precision vector containing the proper 
                     motion in seconds of arc per tropical *century* in right 
                     ascension and declination.
         PARALLAX - N_element vector giving stellar parallax (seconds of arc)
         RAD_VEL  - N_element vector giving radial velocity in km/s
    
          The values of MU_RADEC, PARALLAX, and RADVEL will all be modified
          upon output to contain the values of these quantities in the
          J2000 system.    Values will also be converted to double precision.  
          The parallax and radial velocity will have a very minor influence on 
          the J2000 position.
    
          EPOCH - scalar giving epoch of original observations, default 1950.0d
              This keyword value is only used if the MU_RADEC keyword is not set.
     NOTES:
          The algorithm is taken from the Explanatory Supplement to the 
          Astronomical Almanac 1992, page 184.
          Also see Aoki et al (1983), A&A, 128,263
    
          JPRECESS distinguishes between the following two cases:
          (1) The proper motion is known and non-zero
          (2) the proper motion is unknown or known to be exactly zero (i.e.
                  extragalactic radio sources).   In this case, the algorithm
                  in Appendix 2 of Aoki et al. (1983) is used to ensure that
                  the output proper motion is  exactly zero.    Better precision
                  can be achieved in this case by inputting the EPOCH of the
                  original observations.
    
          The error in using the IDL procedure PRECESS for converting between
          B1950 and J2000 can be up to 12", mainly in right ascension.   If
          better accuracy than this is needed then JPRECESS should be used.
    
    EXAMPLE:
          The SAO catalogue gives the B1950 position and proper motion for the 
          star HD 119288.   Find the J2000 position. 
    
             RA(1950) = 13h 39m 44.526s      Dec(1950) = 8d 38' 28.63''  
             Mu(RA) = -.0259 s/yr      Mu(Dec) = -.093 ''/yr
    
          IDL> mu_radec = 100D* [ -15D*.0259, -0.093 ]
          IDL> ra = ten(13,39,44.526)*15.D 
          IDL> dec = ten(8,38,28.63)
          IDL> jprecess, ra, dec, ra2000, dec2000, mu_radec = mu_radec
          IDL> print, adstring(ra2000, dec2000,2)
                  ===> 13h 42m 12.740s    +08d 23' 17.69"
    
    RESTRICTIONS:
         "When transferring individual observations, as opposed to catalog mean
          place, the safest method is to tranform the observations back to the
          epoch of the observation, on the FK4 system (or in the system that was
          used to to produce the observed mean place), convert to the FK5 system,
          and transform to the the epoch and equinox of J2000.0" -- from the
          Explanatory Supplement (1992), p. 180
    
    REVISION HISTORY:
          Written,    W. Landsman                September, 1992
          Corrected a couple of typos in M matrix   October, 1992
          Vectorized, W. Landsman                   February, 1994
          Implement Appendix 2 of Aoki et al. (1983) for case where proper
          motion unknown or exactly zero     W. Landsman    November, 1994
          Converted to IDL V5.0   W. Landsman   September 1997
          Fixed typo in updating proper motion   W. Landsman   April 1999
          Make sure proper motion is floating point  W. Landsman December 2000
          Use V6.0 notation  W. Landsman Mar 2011
          Converted to python by A. Drlica-Wagner Feb 2014
   """
   if isinstance(ra, ndarray):
      ra = array(ra)
      dec = array(dec)
   else:
      ra = array([ra0])
      dec = array([dec0])
   n = ra.size
    
   if rad_vel is None:   
      rad_vel = zeros(n,dtype=float)
   else:
      if not isinstance(rad_vel, ndarray):
         rad_vel = array([rad_vel],dtype=float)
      if rad_vel.size != n:   
         raise Exception('ERROR - RAD_VEL keyword vector must be of the same length as RA and DEC')
    
   if (mu_radec is not None):   
      if (array(mu_radec).size != 2 * n):   
         raise Exception('ERROR - MU_RADEC keyword (proper motion) be dimensioned (2,' + strtrim(n, 2) + ')')
      mu_radec = mu_radec * 1.
    
   if parallax is None:   
      parallax = zeros(n,dtype=float)
   else:   
      if not isinstance(parallax, ndarray):
         parallax = array([parallax],dtype=float)
    
   if epoch is None:   
      epoch = 1950.0e0
    
   radeg = 180.e0 / pi
   sec_to_radian = 1/radeg/3600.
   #sec_to_radian = lambda x : deg2rad(x/3600.)
    
   m =  array([ 
      array([+0.9999256782e0, +0.0111820610e0, +0.0048579479e0,  \
                -0.000551e0,     +0.238514e0,     -0.435623e0      ]), 
      array([ -0.0111820611e0, +0.9999374784e0, -0.0000271474e0,  \
                 -0.238565e0,     -0.002667e0,      +0.012254e0    ]), 
      array([ -0.0048579477e0, -0.0000271765e0, +0.9999881997e0 , \
                 +0.435739e0,      -0.008541e0,      +0.002117e0   ]), 
      array([ +0.00000242395018e0, +0.00000002710663e0, +0.00000001177656e0, \
                 +0.99994704e0,    +0.01118251e0,    +0.00485767e0 ]), 
      array([ -0.00000002710663e0, +0.00000242397878e0, -0.00000000006582e0, \
                 -0.01118251e0,     +0.99995883e0,    -0.00002714e0]), 
      array([ -0.00000001177656e0, -0.00000000006587e0, 0.00000242410173e0, \
                 -0.00485767e0,   -0.00002718e0,     1.00000956e0  ]) ])
    
   a = 1e-6 * array([ -1.62557e0, -0.31919e0, -0.13843e0])        #in radians
   a_dot = 1e-3 * array([1.244e0, -1.579e0, -0.660e0 ])           #in arc seconds per century
    
   ra_rad = deg2rad(ra)
   dec_rad = deg2rad(dec)
   cosra = cos(ra_rad)
   sinra = sin(ra_rad)
   cosdec = cos(dec_rad)
   sindec = sin(dec_rad)
    
   ra_2000 = ra*0.
   dec_2000 = dec*0.
    
   for i in range(n):
      r0 = array([ cosra[i]*cosdec[i], sinra[i]*cosdec[i], sindec[i] ])
    
      if (mu_radec is None):   
         mu_a = 0.
         mu_d = 0.
      else:
         mu_a = mu_radec[ i, 0]
         mu_d = mu_radec[ i, 1]
    
      #Velocity vector
      r0_dot = array([-mu_a*sinra[i]*cosdec[i] - mu_d*cosra[i]*sindec[i], \
                         mu_a*cosra[i]*cosdec[i] - mu_d*sinra[i]*sindec[i] , \
                         mu_d*cosdec[i] ])  + 21.095e0 * rad_vel[i] * parallax[i] * r0
    
      r1 = r0 - a + ((a * r0).sum())*r0
      r1_dot = r0_dot - a_dot + (( a * r0).sum())*r0
    
      r_1 = concatenate((r1, r1_dot))
      r = transpose(dot(transpose(m),transpose(r_1)))
    
      if mu_radec is None:   
         rr = r[0:3] 
         v =  r[3:6] 
         t = ((epoch - 1950.0e0) - 50.00021e0)/100.0
         rr1 = rr + sec_to_radian*v*t
         x = rr1[0]  ; y = rr1[1]  ; z = rr1[2]  
      else:
         x = r[0]  ; y = r[1]  ; z = r[2]  
         x_dot = r[3]  ; y_dot= r[4]  ;  z_dot = r[5]
    
      r2 = x**2 + y**2 + z**2
      rmag = sqrt( r2 )
      dec_2000[i] = arcsin(z / rmag)
      ra_2000[i] = arctan2(y, x)
    
      if mu_radec is not None:
         mu_radec[i, 0] = ( x*y_dot - y*x_dot) / ( x**2 + y**2)
         mu_radec[i, 1] = ( z_dot* (x**2 + y**2) - z*(x*x_dot + y*y_dot) ) /  \
                     ( r2*sqrt( x**2 + y**2) )
    
      if parallax[i] > 0.:
         rad_vel[i] = ( x*x_dot + y*y_dot + z*z_dot )/ (21.095*parallax[i]*rmag)
         parallax[i] = parallax[i] / rmag
    
   neg = (ra_2000 < 0)
   if neg.any() > 0:
      ra_2000[neg] = ra_2000[neg] + 2.0 * pi
      
   ra_2000 = ra_2000*radeg ; dec_2000 = dec_2000*radeg
    
   if ra.size == 1:
      ra_2000 = ra_2000[0]     ; dec_2000 = dec_2000[0]
    
   return ra_2000, dec_2000


def bprecess(ra0, dec0, mu_radec=None, parallax=None, rad_vel=None, epoch=None):
   """
    NAME:
          BPRECESS
    PURPOSE:
          Precess positions from J2000.0 (FK5) to B1950.0 (FK4)
    EXPLANATION:
          Calculates the mean place of a star at B1950.0 on the FK4 system from
          the mean place at J2000.0 on the FK5 system.
   
    CALLING SEQUENCE:
          bprecess, ra, dec, ra_1950, dec_1950, [ MU_RADEC = , PARALLAX =
                                          RAD_VEL =, EPOCH =   ]
   
    INPUTS:
          RA,DEC - Input J2000 right ascension and declination in *degrees*.
                  Scalar or N element vector
   
    OUTPUTS:
          RA_1950, DEC_1950 - The corresponding B1950 right ascension and
                  declination in *degrees*.    Same number of elements as
                  RA,DEC but always double precision.
   
    OPTIONAL INPUT-OUTPUT KEYWORDS
          MU_RADEC - 2xN element double precision vector containing the proper
                     motion in seconds of arc per tropical *century* in right
                     ascension and declination.
          PARALLAX - N_element vector giving stellar parallax (seconds of arc)
          RAD_VEL  - N_element vector giving radial velocity in km/s
   
          The values of MU_RADEC, PARALLAX, and RADVEL will all be modified
          upon output to contain the values of these quantities in the
          B1950 system.  The parallax and radial velocity will have a very
          minor influence on the B1950 position.
   
          EPOCH - scalar giving epoch of original observations, default 2000.0d
              This keyword value is only used if the MU_RADEC keyword is not set.
    NOTES:
          The algorithm is taken from the Explanatory Supplement to the
          Astronomical Almanac 1992, page 186.
          Also see Aoki et al (1983), A&A, 128,263
   
          BPRECESS distinguishes between the following two cases:
          (1) The proper motion is known and non-zero
          (2) the proper motion is unknown or known to be exactly zero (i.e.
                  extragalactic radio sources).   In this case, the reverse of
                  the algorithm in Appendix 2 of Aoki et al. (1983) is used to
                  ensure that the output proper motion is  exactly zero. Better
                  precision can be achieved in this case by inputting the EPOCH
                  of the original observations.
   
          The error in using the IDL procedure PRECESS for converting between
          B1950 and J1950 can be up to 12", mainly in right ascension.   If
          better accuracy than this is needed then BPRECESS should be used.
   
          An unsystematic comparison of BPRECESS with the IPAC precession
          routine (http://nedwww.ipac.caltech.edu/forms/calculator.html) always
          gives differences less than 0.15".
    EXAMPLE:
          The SAO2000 catalogue gives the J2000 position and proper motion for
          the star HD 119288.   Find the B1950 position.
   
          RA(2000) = 13h 42m 12.740s      Dec(2000) = 8d 23' 17.69''
          Mu(RA) = -.0257 s/yr      Mu(Dec) = -.090 ''/yr
   
          IDL> mu_radec = 100D* [ -15D*.0257, -0.090 ]
          IDL> ra = ten(13, 42, 12.740)*15.D
          IDL> dec = ten(8, 23, 17.69)
          IDL> bprecess, ra, dec, ra1950, dec1950, mu_radec = mu_radec
          IDL> print, adstring(ra1950, dec1950,2)
                  ===> 13h 39m 44.526s    +08d 38' 28.63"
   
    REVISION HISTORY:
          Written,    W. Landsman                October, 1992
          Vectorized, W. Landsman                February, 1994
          Treat case where proper motion not known or exactly zero  November 1994
          Handling of arrays larger than 32767   Lars L. Christensen, march, 1995
          Converted to IDL V5.0   W. Landsman   September 1997
          Fixed bug where A term not initialized for vector input
               W. Landsman        February 2000
         Converted to python 			Sergey Koposov july 2010   
   """

   scal = True
   if isinstance(ra0, ndarray):
      ra = ra0
      dec = dec0
      n = ra.size
      scal = False
   else:
      n = 1
      ra = array([ra0])
      dec = array([dec0])
      
   if rad_vel is None:   
      rad_vel = zeros(n)
   else:
      if not isinstance(rad_vel, ndarray):
         rad_vel = array([rad_vel],dtype=float)
      if rad_vel.size != n:   
         raise Exception('ERROR - RAD_VEL keyword vector must be of the same length as RA and DEC')
   
   if (mu_radec is not None):   
      if (array(mu_radec).size != 2 * n):   
         raise Exception('ERROR - MU_RADEC keyword (proper motion) be dimensioned (2,' + strtrim(n, 2) + ')')
      mu_radec = mu_radec * 1.
   
   if parallax is None:   
      parallax = zeros(n)
   else:   
      if not isinstance(parallax, ndarray):
         parallax = array([parallax],dtype=float)
   
   if epoch is None:   
      epoch = 2000.0e0
   
   radeg = 180.e0 / pi
   sec_to_radian = lambda x : deg2rad(x/3600.)
   
   m = array([array([+0.9999256795e0, -0.0111814828e0, -0.0048590040e0, -0.000551e0, -0.238560e0, +0.435730e0]),
   array([+0.0111814828e0, +0.9999374849e0, -0.0000271557e0, +0.238509e0, -0.002667e0, -0.008541e0]),
   array([+0.0048590039e0, -0.0000271771e0, +0.9999881946e0, -0.435614e0, +0.012254e0, +0.002117e0]),
   array([-0.00000242389840e0, +0.00000002710544e0, +0.00000001177742e0, +0.99990432e0, -0.01118145e0, -0.00485852e0]),
   array([-0.00000002710544e0, -0.00000242392702e0, +0.00000000006585e0, +0.01118145e0, +0.99991613e0, -0.00002716e0]),
   array([-0.00000001177742e0, +0.00000000006585e0, -0.00000242404995e0, +0.00485852e0, -0.00002717e0, +0.99996684e0])])
   
   a_dot = 1e-3 * array([1.244e0, -1.579e0, -0.660e0])           #in arc seconds per century
   
   ra_rad = deg2rad(ra)
   dec_rad = deg2rad(dec)
   cosra = cos(ra_rad)
   sinra = sin(ra_rad)
   cosdec = cos(dec_rad)
   sindec = sin(dec_rad)
   
   dec_1950 = dec * 0.
   ra_1950 = ra * 0.
   
   for i in range(n):
   
   # Following statement moved inside loop in Feb 2000.
      a = 1e-6 * array([-1.62557e0, -0.31919e0, -0.13843e0])        #in radians
      
      r0 = array([cosra[i] * cosdec[i], sinra[i] * cosdec[i], sindec[i]])
      
      if (mu_radec is not None):   
         
         mu_a = mu_radec[i,0]
         mu_d = mu_radec[i,1]
         r0_dot = array([-mu_a * sinra[i] * cosdec[i] - mu_d * cosra[i] * sindec[i], mu_a * cosra[i] * cosdec[i] - mu_d * sinra[i] * sindec[i], mu_d * cosdec[i]]) + 21.095e0 * rad_vel[i] * parallax[i] * r0
         
      else:   
         r0_dot = array([0.0e0, 0.0e0, 0.0e0])
      
      r_0 = concatenate((r0, r0_dot))
      r_1 = transpose(dot(transpose(m), transpose(r_0)))
      
      # Include the effects of the E-terms of aberration to form r and r_dot.
      
      r1 = r_1[0:3]
      r1_dot = r_1[3:6]
      
      if mu_radec is None:   
         r1 = r1 + sec_to_radian ( r1_dot * (epoch - 1950.0e0) / 100. )
         a = a + sec_to_radian ( a_dot * (epoch - 1950.0e0) / 100. )
      
      x1 = r_1[0]   ;   y1 = r_1[1]    ;  z1 = r_1[2]
      rmag = sqrt(x1 ** 2 + y1 ** 2 + z1 ** 2)
      
      
      s1 = r1 / rmag    ; s1_dot = r1_dot / rmag
      
      s = s1
      for j in arange(0, 3):
         r = s1 + a - ((s * a).sum()) * s
         s = r / rmag
      x = r[0]          ; y = r[1]     ;  z = r[2]
      r2 = x ** 2 + y ** 2 + z ** 2
      rmag = sqrt(r2)
      
      if mu_radec is not None:   
         r_dot = s1_dot + a_dot - ((s * a_dot).sum()) * s
         x_dot = r_dot[0]  ; y_dot = r_dot[1]  ;  z_dot = r_dot[2]
         mu_radec[i,0] = (x * y_dot - y * x_dot) / (x ** 2 + y ** 2)
         mu_radec[i,1] = (z_dot * (x ** 2 + y ** 2) - z * (x * x_dot + y * y_dot)) / (r2 * sqrt(x ** 2 + y ** 2))
      
      dec_1950[i] = arcsin(z / rmag)
      ra_1950[i] = arctan2(y, x)
      
      if parallax[i] > 0.:   
         rad_vel[i] = (x * x_dot + y * y_dot + z * z_dot) / (21.095 * parallax[i] * rmag)
         parallax[i] = parallax[i] / rmag
   
   neg = (ra_1950 < 0)
   if neg.any() > 0:   
      ra_1950[neg] = ra_1950[neg] + 2.e0 * pi
   
   ra_1950 = rad2deg(ra_1950)
   dec_1950 = rad2deg(dec_1950)
   
   # Make output scalar if input was scalar
   if scal:
      return ra_1950[0],dec_1950[0]
   else:
      return ra_1950, dec_1950

if __name__ == '__main__':
    from numpy import allclose

    ra  = array([  0., 9., 19., 29., 39., 49., 59., 69., 79., 89.])
    dec = array([ 0., 9., 19., 29., 39., 49., 59., 69., 79., 89.])
    ra2000  = array([0.64069095, 9.6479274, 19.672634, 29.716674, 39.784237, 49.884683, 60.040969, 70.322221, 81.054046, 105.23936])
    dec2000 = array([0.27840945, 9.2747207, 19.262679, 29.242615, 39.215111, 49.180972, 59.141167, 69.096721, 79.048183, 88.965161])
    ra1950  = array([ 359.35928, 8.3527459, 18.328867, 28.285743, 38.219246, 48.120124, 57.965657, 67.687386, 76.962913, 72.909428])
    dec1950 = array([-0.27834947, 8.7248461, 18.736315, 28.755752, 38.782542, 48.815833, 58.854545, 68.89732, 78.94207, 88.955871])

    print "Testing jprecess..."
    test_ra2000, test_dec2000 = jprecess(ra,dec)
    print " RAs:",  allclose(ra2000,test_ra2000)
    print " DECs:", allclose(dec2000,test_dec2000)
    print "Testing bprecess..."
    test_ra1950, test_dec1950 = bprecess(ra,dec)
    print " RAs:",  allclose(ra1950,test_ra1950)
    print " DECs:", allclose(dec1950,test_dec1950)



"""
Tools for making maps of the sky with healpix.
"""

import sys
import re
import gc

import numpy
import healpy
import pyfits

import ugali.utils.projector
from ugali.utils.healpix import superpixel,subpixel,ang2pix,pix2ang,query_disc
from ugali.utils.logger import logger
from ugali.utils.config import Config

############################################################

def surveyPixel(lon, lat, nside_pix, nside_subpix = None):
    """
    Return the set of HEALPix pixels that cover the given coordinates at resolution nside.
    Optionally return the set of subpixels within those pixels at resolution nside_subpix
    """
    pix = numpy.unique(ang2pix)
    if nside_subpix is None:
        return pix
    else:
        subpix_array = []
        for ii in range(0, len(pix)):
            subpix = subpixel(pix[ii], nside_pix, nside_subpix)
            subpix_array.append(subpix)
        return pix, numpy.array(subpix_array)

def inFootprint(config, pixels, nside=None):
    """
    Open each valid filename for the set of pixels and determine the set 
    of subpixels with valid data.
    """
    config = Config(config)
    nside_catalog    = config['coords']['nside_catalog']
    nside_likelihood = config['coords']['nside_likelihood']
    nside_pixel      = config['coords']['nside_pixel']

    if numpy.isscalar(pixels): pixels = numpy.array([pixels])
    if nside is None: nside = nside_likelihood

    filenames = config.getFilenames()
    catalog_pixels = filenames['pix'].compressed()

    inside = numpy.zeros(len(pixels), dtype=bool)
    if not nside_catalog:
        catalog_pix = [0]
    else:
        catalog_pix = superpixel(pixels,nside,nside_catalog)
        catalog_pix = numpy.intersect1d(catalog_pix,catalog_pixels)

    for fnames in filenames[catalog_pix]:
        logger.debug("Loading %s"%filenames['mask_1'])
        subpix_1,val_1 = ugali.utils.skymap.readSparseHealpixMap(fnames['mask_1'],'MAGLIM',construct_map=False)
        logger.debug("Loading %s"%fnames['mask_2'])
        subpix_2,val_2 = ugali.utils.skymap.readSparseHealpixMap(fnames['mask_2'],'MAGLIM',construct_map=False)
        subpix = numpy.intersect1d(subpix_1,subpix_2)
        superpix = numpy.unique(superpixel(subpix,nside_pixel,nside))
        inside |= numpy.in1d(pixels, superpix)
        
    return inside

def footprint(config, nside=None):
    """
    UNTESTED.
    Should return a boolean array representing the pixels in the footprint.
    """
    config = Config(config)
    if nside is None:
        nside = config['coords']['nside_pixel']
    elif nside < config['coords']['nside_catalog']:
        raise Exception('Requested nside=%i is greater than catalog_nside'%nside)
    elif nside > config['coords']['nside_pixel']:
        raise Exception('Requested nside=%i is less than pixel_nside'%nside)
    pix = numpy.arange(healpy.nside2npix(nside), dtype=int)
    return inFootprint(config,pix)


############################################################

def allSkyCoordinates(nside):
    """
    Generate a set of coordinates at the centers of pixels of resolutions nside across the full sky. 
    """
    lon,lat = pix2ang(nside, np.arange(0, healpy.nside2npix(nside)))
    return lon, lat

############################################################

def writeSparseHealpixMap(pix, data_dict, nside, outfile,
                          distance_modulus_array = None,
                          coordsys = 'NULL', ordering = 'NULL',
                          header_dict = None):
    """
    Sparse HEALPix maps are used to efficiently store maps of the sky by only
    writing out the pixels that contain data.

    Three-dimensional data can be saved by supplying a distance modulus array
    which is stored in a separate extension.
    
    coordsys [gal, cel]
    ordering [ring, nest]
    """

    hdul = pyfits.HDUList()

    # Pixel data extension
    columns_array = [pyfits.Column(name = 'PIX',
                                   format = 'K',
                                   array = pix)]

    for key in data_dict.keys():
        if data_dict[key].shape[0] != len(pix):
            logger.warning('First dimension of column %s (%i) does not match number of pixels (%i).'%(key,
                                                                                                      data_dict[key].shape[0],
                                                                                                      len(pix)))
        
        if len(data_dict[key].shape) == 1:
            columns_array.append(pyfits.Column(name = key,
                                               format = 'E',
                                               array = data_dict[key]))
        elif len(data_dict[key].shape) == 2:
            columns_array.append(pyfits.Column(name = key,
                                               format = '%iE'%(data_dict[key].shape[1]),
                                               array = data_dict[key]))
        else:
            logger.warning('Unexpected number of data dimensions for column %s.'%(key))
    
    hdu_pix_data = pyfits.new_table(columns_array)
    hdu_pix_data.header.update('NSIDE', nside)
    hdu_pix_data.header.update('COORDSYS', coordsys.upper())
    hdu_pix_data.header.update('ORDERING', ordering.upper())
    hdu_pix_data.header.update(header_dict)
    hdu_pix_data.name = 'PIX_DATA'
    hdul.append(hdu_pix_data)

    # Distance modulus extension
    if distance_modulus_array is not None:
        hdu_distance_modulus = pyfits.new_table([pyfits.Column(name = 'DISTANCE_MODULUS',
                                                               format = 'E',
                                                               array = distance_modulus_array)])
        hdu_distance_modulus.name = 'DISTANCE_MODULUS'
        hdul.append(hdu_distance_modulus)

    hdul.writeto(outfile, clobber = True)
    
############################################################

def readSparseHealpixMap(infile, field, extension='PIX_DATA', default_value=healpy.UNSEEN, construct_map=True):
    """
    Open a sparse HEALPix map fits file.
    Convert the contents into a HEALPix map or simply return the contents.
    Flexibility to handle 
    """
    reader = pyfits.open(infile,memmap=False)
    nside = reader[extension].header['NSIDE']

    # Trying to fix avoid a memory leak
    pix = numpy.array(reader[extension].data.field('PIX'),copy=True)
    value = numpy.array(reader[extension].data.field(field),copy=True)
    reader.close()
    
    if construct_map:
        if len(value.shape) == 1:
            map = default_value * numpy.ones(healpy.nside2npix(nside))
            map[pix] = value
        else:
            map = default_value * numpy.ones([value.shape[1], healpy.nside2npix(nside)])
            for ii in range(0, value.shape[1]):
                map[ii][pix] = numpy.take(value, [ii], axis=1)
        ret = map
    else:
        if len(value.shape) == 1:
            ret = (pix,value)
        else:
            ret = (pix,value.transpose())
    return ret

############################################################

def readSparseHealpixMaps(infiles, field, extension='PIX_DATA', default_value=healpy.UNSEEN, construct_map=True):
    """
    Read multiple sparse healpix maps and output the results
    identically to a single file read.
    """
    if isinstance(infiles,basestring): infiles = [infiles]

    pix_array   = []
    value_array = []

    # Create a map based on the first file in the list
    map = readSparseHealpixMap(infiles[0], field, extension=extension, default_value=healpy.UNSEEN, construct_map=True)

    for ii in range(0, len(infiles)):
        logger.debug('(%i/%i) %s'%(ii+1, len(infiles), infiles[ii]))
        pix_array_current, value_array_current = readSparseHealpixMap(infiles[ii], field,
                                                                      extension=extension,
                                                                      construct_map=False)
        pix_array.append(pix_array_current)
        value_array.append(value_array_current)
        map[pix_array[ii]] = value_array[ii]

    # Check to see whether there are any conflicts
    pix_master = numpy.concatenate(pix_array)
    value_master = numpy.concatenate(value_array)

    n_conflicting_pixels = len(pix_master) - len(numpy.unique(pix_master)) 
    if n_conflicting_pixels != 0:
        logger.warning('%i conflicting pixels during merge.'%(n_conflicting_pixels))

    if construct_map:
        return map
    else:
        if n_conflicting_pixels == 0:
            pix_master = numpy.sort(pix_master)
            return pix_master, map[pix_master]
        else:
            pix_valid = numpy.nonzero(map != default_value)[0]
            return pix_valid, map[pix_valid]

############################################################

def mergeSparseHealpixMaps(infiles, outfile=None,
                           pix_data_extension='PIX_DATA',
                           pix_field='PIX',
                           distance_modulus_extension='DISTANCE_MODULUS',
                           distance_modulus_field='DISTANCE_MODULUS',
                           default_value=healpy.UNSEEN):
    """
    Use the first infile to determine the basic contents to expect for the other files.
    """
    # Setup
    if isinstance(infiles,basestring): infiles = [infiles]
    
    distance_modulus_array = None
    pix_array = []
    data_dict = {}

    reader = pyfits.open(infiles[0])
    nside = reader[pix_data_extension].header['NSIDE']

    for ii in range(0, len(reader)):
        if reader[ii].name == distance_modulus_extension:
            distance_modulus_array = reader[distance_modulus_extension].data.field(distance_modulus_field)

    for key in reader[pix_data_extension].data.names:
        if key == pix_field:
            continue
        data_dict[key] = []
        #if distance_modulus_array is None:
        #    data_dict[key] = default_value * numpy.ones(healpy.nside2npix(nside))
        #else:
        #    data_dict[key] = default_value * numpy.ones([len(distance_modulus_array),
        #                                                 healpy.nside2npix(nside)])
    reader.close()

    # Now loop over the infiles

    for ii in range(0, len(infiles)):
        logger.debug('(%i/%i) %s'%(ii+1, len(infiles), infiles[ii]))

        reader = pyfits.open(infiles[ii])
        distance_modulus_array_current = numpy.array(reader[distance_modulus_extension].data.field(distance_modulus_field),copy=True)
        if not numpy.array_equal(distance_modulus_array_current,distance_modulus_array):
            logger.warning("Distance moduli do not match; skipping...")
            continue
        reader.close()

        pix_array_current = readSparseHealpixMap(infiles[ii], pix_field,
                                                 extension=pix_data_extension, construct_map=False)[0]
        pix_array.append(pix_array_current)

        for key in data_dict.keys():
            value_array_current = readSparseHealpixMap(infiles[ii], key,
                                                       extension=pix_data_extension, construct_map=False)[1]
            data_dict[key].append(value_array_current)
            #if distance_modulus_array is None:
            #    data_dict[key][pix_array_current] = value
            #else:
            #    for jj in range(0, len(distance_modulus_array)):
            #        data_dict[key][jj] = value[jj]

        gc.collect()

    pix_master = numpy.concatenate(pix_array)
    n_conflicting_pixels = len(pix_master) - len(numpy.unique(pix_master)) 
    if n_conflicting_pixels != 0:
        logger.warning('%i conflicting pixels during merge.'%(n_conflicting_pixels))

    for key in data_dict.keys():
        if distance_modulus_array is not None:
            data_dict[key] = numpy.concatenate(data_dict[key], axis=1).transpose()
        else:
            data_dict[key] = numpy.concatenate(data_dict[key])

    if outfile is not None:
        writeSparseHealpixMap(pix_master, data_dict, nside, outfile,
                              distance_modulus_array=distance_modulus_array,
                              coordsys='NULL', ordering='NULL')
    else:
        return data_dict


############################################################

def mergeLikelihoodFiles(infiles, lkhdfile, roifile):
    mergeSparseHealpixMaps(infiles,lkhdfile)

    ext='PIX_DATA'
    keys=['STELLAR','NINSIDE','NANNULUS']
    nside = pyfits.open(infiles[0])[ext].header['LKDNSIDE']

    pix_array = []
    data_dict = dict([(k,[]) for k in keys])
    for ii in range(0, len(infiles)):
        logger.debug('(%i/%i) %s'%(ii+1, len(infiles), infiles[ii]))
        reader = pyfits.open(infiles[ii])
        pix_array.append(reader[ext].header['LKDPIX'])
        for key in data_dict.keys():
            data_dict[key].append(reader[ext].header[key])
        
    pix_array = numpy.array(pix_array)
    for key in data_dict.keys():
        data_dict[key] = numpy.array(data_dict[key])
    writeSparseHealpixMap(pix_array, data_dict, nside, roifile)

############################################################

def randomPositions(input, nside_pix, n=1):
    """
    Generate n random positions within a full HEALPix mask of booleans, or a set of (lon, lat) coordinates.

    nside_pix is meant to be at coarser resolution than the input mask or catalog object positions
    so that gaps from star holes, bleed trails, cosmic rays, etc. are filled in. 
    Return the longitude and latitude of the random positions and the total area (deg^2).

    Probably there is a faster algorithm, but limited much more by the simulation and fitting time
    than by the time it takes to generate random positions within the mask.
    """
    input = numpy.array(input)
    if len(input.shape) == 1:
        subpix = numpy.nonzero(input)[0] # All the valid pixels in the mask at the NSIDE for the input mask
        lon, lat = pix2ang(healpy.npix2nside(len(input)), subpix)
    elif len(input.shape) == 2:
        lon, lat = input[0], input[1] # All catalog object positions
    else:
        logger.warning('Unexpected input dimensions for skymap.randomPositions')
    pix = surveyPixel(lon, lat, nside_pix)

    # Area with which the random points are thrown
    area = len(pix) * healpy.nside2pixarea(nside_pix, degrees=True)
    
    lon = []
    lat = []
    for ii in range(0, n):
        # Choose an unmasked pixel at random, which is OK because HEALPix is an equal area scheme
        pix_ii = pix[numpy.random.randint(0, len(pix))]
        lon_ii, lat_ii = ugali.utils.projector.pixToAng(nside_pix, pix_ii)
        projector = ugali.utils.projector.Projector(lon_ii, lat_ii)

        inside = False
        while not inside:
            # Apply random offset
            arcminToDegree = 1 / 60.
            resolution = arcminToDegree * healpy.nside2resol(nside_pix, arcmin=True)
            x = 2. * (numpy.random.rand() - 0.5) * resolution # Using factor 2 to be conservative
            y = 2. * (numpy.random.rand() - 0.5) * resolution
            
            lon_candidate, lat_candidate = projector.imageToSphere(x, y)

            # Make sure that the random position does indeed fall within the randomly selected pixel 
            if ugali.utils.projector.angToPix(nside_pix, lon_candidate, lat_candidate) == pix_ii:
                inside = True
                                    
        lon.append(lon_candidate)
        lat.append(lat_candidate)

    return numpy.array(lon), numpy.array(lat), area

############################################################




#!/usr/bin/env python
"""
Script to automate the process of generating an isochrone library using the Dotter isochrones.

Download isochrones from
http://stellar.dartmouth.edu/models/isolf_new.html
"""

import urllib2
import os
import sys
import numpy

import ugali.utils.logger
import ugali.utils.shell

############################################################

def getDotterIsochrone(age, z, outdir, afe='+0.4', hel='Y=0.245+1.5*Z', clr='DECam', clobber=False):
    """
    See Vargas et al. 2013 for the distribution of alpha elements in
    dSphs: http://adsabs.harvard.edu/abs/2013ApJ...767..134V
    
    Josh Simon remarks: For stars at [Fe/H] > -2, [a/Fe] tends to be
    around zero. [Note, though, that this paper does not attempt to do
    any membership classification, it just accepts the lists from
    Simon & Geha 2007.  I suspect now that we were not sufficiently
    conservative on selecting members in those early days, and so some
    of the relatively metal-rich stars may in fact be foreground Milky
    Way stars.]  More metal-poor stars tend to average more like
    [a/Fe] = 0.4-0.5.  Fig. 5 of Frebel et al. (2014) shows similar
    plots for individual elements from high-resolution spectra.  Given
    these data, plus the empirical fact that the mean metallicities of
    the ultra-faint dwarfs are almost universally [Fe/H] < -2, I guess
    I would say [a/Fe] = 0.3 is probably the best compromise.
    """

    # KCB: This check won't work if the isochrone files have to be nenamed
    outfile = '%s/iso_a%.1f_z%.5f_temp.dat'%(outdir, age, z)
    if os.path.exists(outfile) and not clobber:
        ugali.utils.logger.logger.warning('Found %s; skipping...'%(outfile))
        return

    ugali.utils.logger.logger.info('Downloading isochrone: %s (age=%.2fGyr, metallicity=%g)'%(os.path.basename(outfile), age, z))

    ##### Hard-coded options from the Dotter web interface -- do not use blindly #####

    dict_clr = {'SDSS ugriz': '11',
                'PanSTARRS': '12',
                'DECam': '14'}
    if clr not in dict_clr.keys():
        ugali.utils.logger.logger.error('Photometric system %s not in available options: %s'%(clr, ', '.join(dict_clr.keys())))
        sys.exit()

    dict_afe = {'-0.2': '1',
                '0 (scaled-solar)': '2',
                '+0.2': '3',
                '+0.4': '4',
                '+0.6': '5',
                '+0.8': '6'}
    if afe not in dict_afe.keys():
        ugali.utils.logger.logger.error('[alpha/Fe] ratio %s not in available options: %s'%(afe, ', '.join(dict_afe.keys())))
        sys.exit()

    dict_hel = {'Y=0.245+1.5*Z': '1',
                'Y=0.33': '2',
                'Y=0.40': '3'}
    if hel not in dict_hel.keys():
        ugali.utils.logger.logger.error('Helium abundance %s not in available options: %s'%(hel, ', '.join(dict_hel.keys())))
        sys.exit()

    #####

    z_solar = 0.02 # Approximately the Dotter convention
    #z_solar = 0.0163 # Dotter convention
    feh = numpy.log10(z / z_solar)
    #print feh
    query = 'http://stellar.dartmouth.edu/models/isolf_new.php?int=1&out=1&age=%.1f&feh=%.5f&hel=%i&afe=%i&clr=%i&flt=&bin=&imf=1&pls=&lnm=&lns='%(age, 
                                                                                                                                                   feh, 
                                                                                                                                                   int(dict_hel[hel]), 
                                                                                                                                                   int(dict_afe[afe]), 
                                                                                                                                                   int(dict_clr[clr]))
    response = urllib2.urlopen(query)
    page_source = response.read()
    isochrone_id = page_source.split('tmp/tmp')[-1].split('.iso')[0]
    infile = 'http://stellar.dartmouth.edu/models/tmp/tmp%s.iso'%(isochrone_id)
    command = 'wget -q %s -O %s'%(infile, outfile)
    os.system(command)

    # Now rename the output file based on Z effective
    reader = open(outfile)
    lines = reader.readlines()
    reader.close()
    z_eff = float(lines[3].split()[4])

    outfile_new = '%s/iso_a%.1f_z%.5f.dat'%(outdir, age, z_eff)
    #outfile = 'iso_a%.1f_z%.5f.dat'%(age, z)
    #command = 'mv tmp%s.iso %s'%(isochrone_id, outfile_new)
    command = 'mv %s %s'%(outfile, outfile_new)

    #print command
    os.system(command)

############################################################

if __name__ == "__main__":
    #import ugali.utils.parser
    #description = "Download Dotter isochrones"
    #parser = ugali.utils.parser.Parser(description=description)

    #from ugali.utils.config import Config
    #config = Config(opts.config)

    #outdir = '/Users/keithbechtol/Documents/DES/projects/mw_substructure/des/isochrones/dotter_v3/'
    outdir = '/Users/keithbechtol/Documents/DES/projects/mw_substructure/des/isochrones/dotter_v4/'

    if outdir is None: outdir = './'
    ugali.utils.shell.mkdir(outdir)

    # Range used for PARSEC isochrone library
    #z_array = 10**numpy.linspace(numpy.log10(1.e-4), numpy.log10(1.e-2), 33)
    #age_array = numpy.arange(1., 13.5 + 0.1, 0.1)
    # Going to higher metallicity, [Fe/H] = -2.49
    z_array = 10**numpy.linspace(numpy.log10(6.5e-5), numpy.log10(5.e-3), 33)
    age_array = numpy.arange(6., 13.5 + 0.1, 0.1)
    
    # Test
    #getDotterIsochrone(12.2, 1.1e-4, outdir=outdir)
    #print '\nIf this were not a test, this script would download the following isochrone grid...'

    print '\nAges:'
    print age_array
    print '\nMetallicities:'
    print z_array

    count = 0
    for z in z_array:
        for age in age_array:
            print '(%i/%i)'%(count, len(age_array) * len(z_array))
            getDotterIsochrone(age, z, outdir=outdir)
            count += 1

############################################################



#!/usr/bin/env python
"""
Module for simulation.

"""

import copy

import numpy
import numpy as np
import scipy.interpolate
import pyfits
import healpy
import numpy.lib.recfunctions as recfuncs


import ugali.observation.catalog
import ugali.observation.mask
import ugali.observation.roi
import ugali.utils.projector
import ugali.utils.stats
import ugali.analysis.scan

from ugali.utils.projector import gal2cel, cel2gal, sr2deg, mod2dist
from ugali.utils.healpix import ang2pix, pix2ang
from ugali.utils.logger import logger
from ugali.utils.config import Config

class Generator:
    """
    Class for generating the parameters of the simulation.
    """
    def __init__(self,config, seed=None):
        self.config = Config(config)
        self.seed = seed
        if self.seed is not None: np.random.seed(self.seed)

    def generate(self, size=1):
        params = dict(self.config['simulate']['params'])
        dtype = [(n,'>f4') for n in params.keys()]
        data = np.zeros(size,dtype=dtype)

        lon,lat = params.pop('lon'),params.pop('lat')
        data['lon'],data['lat'] = self.sky(lon,lat,size)

        for key,value in params.items():
            if value[-1] == 'linear':
                data[key] = self.linear(value[0],value[1],size)
            elif value[-1] == 'log':
                data[key] = self.logarithmic(value[0],value[1],size)
            else:
                raise Exception('...')
        return data

    def sky(self,lon=None,lat=None,size=1):
        logger.info("Generating %i random points..."%size)
        # Random longitue and latitude
        lon,lat = ugali.utils.stats.sky(lon,lat,size=10*size)
        # Random healpix coordinates inside footprint
        nside_pixel = self.config['coords']['nside_pixel']
        pixels = ang2pix(nside_pixel,lon,lat)
        if np.unique(pixels).size > 1:
            inside = ugali.utils.skymap.inFootprint(self.config,pixels,nside=nside_pixel)
        else:
            inside = np.ones(len(pixels),dtype=bool)
        return lon[inside][:size],lat[inside][:size]
        
    def linear(self,low,high,size):
        return np.random.uniform(low,high,size)
    
    def logarithmic(self,low,high,size):
        if low==0 and high==0:
            logger.warning("Can't sample logarithmically with boundary of zero.")
            return np.zeros(size)
        return 10**np.random.uniform(np.log10(low),np.log10(high),size)

    def detectability(self,**kwargs):
        """
        An a priori detectability proxy.
        """
        distance_modulus = kwargs.get('distance_modulus')
        distance = mod2dist(distance_modulus)
        stellar_mass = kwargs.get('stellar_mass')
        extension = kwargs.get('extension')

        # Normalized to 10^3 Msolar at mod=18
        norm = 10**3/mod2dist(18)**2
        detect = stellar_mass / distance**2
        detect /= norm

    def write(self, filename, data=None):
        if data is None: data = self.results
        logger.info("Writing %s..."%filename)
        if filename.endswith('.npy'):
            np.save(filename,data)
        elif filename.endswith('.fits'):
            # Copies data, so be careful..
            out = np.rec.array(data)
            out.dtype.names = np.char.upper(out.dtype.names)
            hdu = pyfits.new_table(out)
            hdu.writeto(filename,clobber=True)
        elif filename.endswith('.txt') or filename.endswith('.dat'):
            np.savetxt(filename,data)
        elif filename.endswith('.txt') or filename.endswith('.dat'):
            np.savetxt(filename,data)
        else:
            raise Exception('Unrecognized file extension: %s'%filename)

    def run(self, outfile=None, size=None):
        if size is None: size = self.config['simulate']['size']
        data = self.generate(size)
        
        dtype=[('kernel','S18'),('ts','>f4'),('fit_kernel','S18'),('fit_ts','>f4'),
               ('fit_mass','>f4'),('fit_mass_err','>f4'),
               ('fit_distance','>f4'),('fit_distance_err','>f4')]
        results = np.array(np.nan*np.ones(size),dtype=dtype)
        results = recfuncs.merge_arrays([data,results],flatten=True,asrecarray=False,usemask=False)
        self.results = results

        if outfile: self.write(outfile,results)

        for i,d in enumerate(data): 
            params = dict(zip(data.dtype.names,d))
            lon,lat = params['lon'],params['lat']
            distance_modulus = params['distance_modulus']

            logger.info('\n(%i/%i); (lon, lat) = (%.2f, %.2f)'%(i+1,len(data),lon,lat))
            roi = ugali.analysis.loglike.createROI(self.config,lon,lat)
            mask = ugali.analysis.loglike.createMask(self.config,roi)
            isochrone = ugali.analysis.loglike.createIsochrone(self.config)
            kernel = ugali.analysis.loglike.createKernel(self.config,lon,lat)
            pix = roi.indexTarget(lon,lat)

            simulator = Simulator(self.config,roi)
            #catalog   = simulator.simulate(seed=self.seed, **params)
            catalog   = simulator.simulate(**params)
            #print "Catalog annulus contains:",roi.inAnnulus(simulator.catalog.lon,simulator.catalog.lat).sum()
            logger.info("Simulated catalog annulus contains %i stars"%roi.inAnnulus(catalog.lon,catalog.lat).sum())

            if len(catalog.lon) < 1000:
                logger.error("Simulation contains too few objects; skipping...")
                continue

            """
            like = ugali.analysis.loglike.LogLikelihood(self.config, roi, mask, catalog, isochrone, kernel)
            like.set_params(distance_modulus=params['distance_modulus'])
            like.sync_params()
            results[i]['ts'] = 2*like.fit_richness()[0]
            print 'TS=',results[i]['ts'] 
            
            like2 = ugali.analysis.loglike.LogLikelihood(self.config, roi, mask, simulator.catalog, isochrone, kernel)
            like2.set_params(distance_modulus=params['distance_modulus'])
            like2.sync_params()
            print 'TS=',2*like2.fit_richness()[0]
            """
            #return simulator,like,like2

            # Index of closest distance modulus
            grid = ugali.analysis.scan.GridSearch(self.config,roi,mask,catalog,isochrone,kernel)

            self.catalog = catalog
            self.simulator = simulator
            self.grid = grid
            self.loglike = self.grid.loglike

            # ADW: Should allow fit_distance to float in order to model search procedure
            #fit_distance = float(distance_modulus)
            distance_idx = np.fabs(grid.distance_modulus_array-params['distance_modulus']).argmin()
            fit_distance = grid.distance_modulus_array[distance_idx]
            grid.search(coords=(lon,lat),distance_modulus=fit_distance)

            logger.info(str(self.loglike))

            mle = grid.mle()
            results[i]['kernel'] = simulator.kernel.name
            results[i]['fit_kernel'] = grid.loglike.kernel.name
            results[i]['ts'] = 2*grid.log_likelihood_sparse_array[distance_idx][pix]
            results[i]['fit_ts'] = 2*np.max(grid.log_likelihood_sparse_array[:,pix])
            results[i]['fit_mass'] = grid.stellar_mass_conversion*mle['richness']
            results[i]['fit_distance'] = fit_distance #mle['distance_modulus']

            err = grid.err()
            richness_err = (err['richness'][1]-err['richness'][0])/2.
            results[i]['fit_mass_err'] = grid.stellar_mass_conversion*richness_err

            distance_modulus_err = (err['distance_modulus'][1]-err['distance_modulus'][0])/2.
            results[i]['fit_distance_err'] = distance_modulus_err

            for d in dtype:
                logger.info('\t%s: %s'%(d[0], results[i][d[0]]))

            if i%self.config['simulate']['save']==0 and outfile: 
                self.write(outfile,results)

        if outfile: self.write(outfile,results)
            
        return results

############################################################

class Simulator(object):
    """
    Class for simulating catalog data.
    """
    
    def __init__(self, config, roi, **kwargs):
        self.config = ugali.utils.config.Config(config)
        self.roi = roi
        #np.random.seed(0)

        params = dict(self.config)
        if self.config['simulate'].get('isochrone') is None:
            params['simulate']['isochrone'] = params['isochrone']
        if self.config['simulate'].get('kernel') is None:
            params['simulate']['kernel'] = params['kernel']

        self.isochrone = ugali.analysis.loglike.createIsochrone(params)
        self.kernel = ugali.analysis.loglike.createKernel(params['simulate'],self.roi.lon,self.roi.lat)

        self.mask = ugali.analysis.loglike.createMask(self.config,self.roi)

        self._create_catalog(kwargs.get('catalog'))

        self.photo_err_1,self.photo_err_2 = self.mask.photo_err_1,self.mask.photo_err_2
        #self._photometricErrors()
        self._setup_subpix()
        #self._setup_cmd()

    def _create_catalog(self,catalog=None):
        """
        Bundle it.
        """
        if catalog is None:
            catalog = ugali.analysis.loglike.createCatalog(self.config,self.roi)
        cut = self.mask.restrictCatalogToObservableSpace(catalog)
        self.catalog = catalog.applyCut(cut)
        
    def _photometricErrors(self, n_per_bin=100, plot=False):
        """
        Realistic photometric errors estimated from catalog objects and mask.
        Extend below the magnitude threshold with a flat extrapolation.
        """

        self.catalog.spatialBin(self.roi)

        if len(self.catalog.mag_1) < n_per_bin:
            logger.warning("Catalog contains fewer objects than requested to calculate errors.")
            n_per_bin = int(len(self.catalog.mag_1) / 3)

        # Band 1
        mag_1_thresh = self.mask.mask_1.mask_roi_sparse[self.catalog.pixel_roi_index] - self.catalog.mag_1
        sorting_indices = numpy.argsort(mag_1_thresh)
        mag_1_thresh_sort = mag_1_thresh[sorting_indices]
        mag_err_1_sort = self.catalog.mag_err_1[sorting_indices]

        # ADW: Can't this be done with numpy.median(axis=?)
        mag_1_thresh_medians = []
        mag_err_1_medians = []
        for i in range(0, int(len(mag_1_thresh) / float(n_per_bin))):
            mag_1_thresh_medians.append(numpy.median(mag_1_thresh_sort[n_per_bin * i: n_per_bin * (i + 1)]))
            mag_err_1_medians.append(numpy.median(mag_err_1_sort[n_per_bin * i: n_per_bin * (i + 1)]))
        
        if mag_1_thresh_medians[0] > 0.:
            mag_1_thresh_medians = numpy.insert(mag_1_thresh_medians, 0, -99.)
            mag_err_1_medians = numpy.insert(mag_err_1_medians, 0, mag_err_1_medians[0])
        
        self.photo_err_1 = scipy.interpolate.interp1d(mag_1_thresh_medians, mag_err_1_medians,
                                                      bounds_error=False, fill_value=mag_err_1_medians[-1])

        # Band 2
        mag_2_thresh = self.mask.mask_2.mask_roi_sparse[self.catalog.pixel_roi_index] - self.catalog.mag_2
        sorting_indices = numpy.argsort(mag_2_thresh)
        mag_2_thresh_sort = mag_2_thresh[sorting_indices]
        mag_err_2_sort = self.catalog.mag_err_2[sorting_indices]

        mag_2_thresh_medians = []
        mag_err_2_medians = []
        for i in range(0, int(len(mag_2_thresh) / float(n_per_bin))):
            mag_2_thresh_medians.append(numpy.median(mag_2_thresh_sort[n_per_bin * i: n_per_bin * (i + 1)]))
            mag_err_2_medians.append(numpy.median(mag_err_2_sort[n_per_bin * i: n_per_bin * (i + 1)]))

        if mag_2_thresh_medians[0] > 0.:
            mag_2_thresh_medians = numpy.insert(mag_2_thresh_medians, 0, -99.)
            mag_err_2_medians = numpy.insert(mag_err_2_medians, 0, mag_err_2_medians[0])
        
        self.photo_err_2 = scipy.interpolate.interp1d(mag_2_thresh_medians, mag_err_2_medians,
                                                      bounds_error=False, fill_value=mag_err_2_medians[-1])


    def _setup_subpix(self,nside=2**16):
        """
        Subpixels for random position generation.
        """
        # Only setup once...
        if hasattr(self,'subpix'): return

        # Simulate over full ROI
        self.roi_radius  = self.config['coords']['roi_radius']

        # Setup background spatial stuff
        logger.info("Setup subpixels...")
        self.nside_pixel = self.config['coords']['nside_pixel']
        self.nside_subpixel = self.nside_pixel * 2**4 # Could be config parameter
        epsilon = np.degrees(healpy.max_pixrad(self.nside_pixel)) # Pad roi radius to cover edge healpix
        subpix = ugali.utils.healpix.query_disc(self.nside_subpixel,self.roi.vec,self.roi_radius+epsilon)
        superpix = ugali.utils.healpix.superpixel(subpix,self.nside_subpixel,self.nside_pixel)
        self.subpix = subpix[np.in1d(superpix,self.roi.pixels)]

    def _setup_cmd(self,mode='cloud-in-cells'):
        """
        The purpose here is to create a more finely binned
        background CMD to sample from.
        """
        # Only setup once...
        if hasattr(self,'bkg_lambda'): return

        logger.info("Setup color...")
        # In the limit theta->0: 2*pi*(1-cos(theta)) -> pi*theta**2
        # (Remember to convert from sr to deg^2) 
        #solid_angle_roi = sr2deg(2*np.pi*(1-np.cos(np.radians(self.roi_radius))))
        solid_angle_roi = self.roi.area_pixel*len(self.roi.pixels)

        # Large CMD bins cause problems when simulating
        config = Config(self.config) 
        config['color']['n_bins'] *= 5 #10
        config['mag']['n_bins']   *= 1 #2
        #config['mask']['minimum_solid_angle'] = 0
        roi = ugali.analysis.loglike.createROI(config,self.roi.lon,self.roi.lat)
        mask = ugali.analysis.loglike.createMask(config,roi)

        self.bkg_centers_color  = roi.centers_color
        self.bkg_centers_mag    = roi.centers_mag

        # Background CMD has units: [objs / deg^2 / mag^2]
        cmd_background = mask.backgroundCMD(self.catalog,mode)
        
        self.bkg_lambda=cmd_background*solid_angle_roi*roi.delta_color*roi.delta_mag
        np.sum(self.bkg_lambda)

        # Clean up 
        del config, roi, mask


    def toy_background(self,mc_source_id=2,seed=None):
        """
        Quick uniform background generation.
        """

        logger.info("Running toy background simulation...")
        size = 20000
        nstar = np.random.poisson(size)
        #np.random.seed(0)
        logger.info("Simulating %i background stars..."%nstar)

        ### # Random points from roi pixels
        ### idx = np.random.randint(len(self.roi.pixels)-1,size=nstar)
        ### pix = self.roi.pixels[idx]

        # Random points drawn from subpixels
        logger.info("Generating uniform positions...")
        idx = np.random.randint(0,len(self.subpix)-1,size=nstar)
        lon,lat = pix2ang(self.nside_subpixel,self.subpix[idx])

        pix = ang2pix(self.nside_pixel, lon, lat)
        lon,lat = pix2ang(self.nside_pixel,pix)

        # Single color
        #mag_1 = 19.05*np.ones(len(pix))
        #mag_2 = 19.10*np.ones(len(pix))

        # Uniform in color
        logger.info("Generating uniform CMD...")
        mag_1 = np.random.uniform(self.config['mag']['min'],self.config['mag']['max'],size=nstar)
        color = np.random.uniform(self.config['color']['min'],self.config['color']['max'],size=nstar)

        mag_2 = mag_1 - color

        # There is probably a better way to do this step without creating the full HEALPix map
        mask = -1. * numpy.ones(healpy.nside2npix(self.nside_pixel))
        mask[self.roi.pixels] = self.mask.mask_1.mask_roi_sparse
        mag_lim_1 = mask[pix]
        mask = -1. * numpy.ones(healpy.nside2npix(self.nside_pixel))
        mask[self.roi.pixels] = self.mask.mask_2.mask_roi_sparse
        mag_lim_2 = mask[pix]
        
        #mag_err_1 = 1.0*np.ones(len(pix))
        #mag_err_2 = 1.0*np.ones(len(pix))
        mag_err_1 = self.photo_err_1(mag_lim_1 - mag_1)
        mag_err_2 = self.photo_err_2(mag_lim_2 - mag_2)
        mc_source_id = mc_source_id * numpy.ones(len(mag_1))

        select = (mag_lim_1>mag_1)&(mag_lim_2>mag_2)
        
        hdu = ugali.observation.catalog.makeHDU(self.config,mag_1[select],mag_err_1[select],
                                                mag_2[select],mag_err_2[select],
                                                lon[select],lat[select],mc_source_id[select])
        catalog = ugali.observation.catalog.Catalog(self.config, data=hdu.data)
        return catalog


    def background(self,mc_source_id=2,seed=None):
        """
        Create a simulation of the background stellar population.
        Because some stars have been clipped to generate the CMD,
        this function tends to slightly underestimate (~1%) the 
        background as compared to the true catalog.

        The simulation of background object colors relies on the
        data-derived CMD. As such, it is a binned random generator
        and thus has some fundamental limitations.
        - The expected number of counts per bin is drawn ra

        There are a few limitations of this procedure:
        - Colors are drawn from the CMD of the background annulus
        - The number of stars per CMD bin is randomized according to the CMD
        - The colors/mags are then uniformly distributed within the bin
        - This leads to trouble with large bins when the cloud-in-cells 
        algorithm is applied to the simulated data
        - The positions are chosen randomly over the spherical cap of the ROI
        - Objects that are outside of the 

        WARNING: The cloud-in-cells method of generating
        the CMD leads to some difficulties since it disperses
        objects from high-density zones to low density zones.

        - Magnitudes are not randomized according to their errors
        """
        if seed is not None: np.random.seed(seed)
        self._setup_cmd()

        # Randomize the number of stars per bin according to Poisson distribution
        nstar_per_bin = numpy.random.poisson(lam=self.bkg_lambda)
        nstar = nstar_per_bin.sum()

        logger.info("Simulating %i background stars..."%nstar)

        if not self.config['simulate'].get('uniform'):
            logger.info("Generating colors from background CMD.")

            # Distribute the stars within each CMD bin
            delta_color = self.bkg_centers_color[1]-self.bkg_centers_color[0]
            delta_mag   = self.bkg_centers_mag[1]-self.bkg_centers_mag[0]

            # Distribute points within each color-mag bins
            xx,yy = np.meshgrid(self.bkg_centers_color,self.bkg_centers_mag)
            color = numpy.repeat(xx.flatten(),repeats=nstar_per_bin.flatten())
            color += numpy.random.uniform(-delta_color/2.,delta_color/2.,size=nstar)
            mag_1 = numpy.repeat(yy.flatten(),repeats=nstar_per_bin.flatten())
            mag_1 += numpy.random.uniform(-delta_mag/2.,delta_mag/2.,size=nstar)
        else:
            # Uniform color-magnitude distribution
            logger.info("Generating uniform CMD.")
            mag_1 = np.random.uniform(self.config['mag']['min'],self.config['mag']['max'],size=nstar)
            color = np.random.uniform(self.config['color']['min'],self.config['color']['max'],size=nstar)

        mag_2 = mag_1 - color

        # Random points drawn from healpix subpixels
        logger.info("Generating uniform positions...")
        idx = np.random.randint(0,len(self.subpix)-1,size=nstar)
        lon,lat = pix2ang(self.nside_subpixel,self.subpix[idx])

        nside_pixel = self.nside_pixel
        pix = ang2pix(nside_pixel, lon, lat)

        # There is probably a better way to do this step without creating the full HEALPix map
        mask = -1. * numpy.ones(healpy.nside2npix(nside_pixel))
        mask[self.roi.pixels] = self.mask.mask_1.mask_roi_sparse
        mag_lim_1 = mask[pix]
        mask = -1. * numpy.ones(healpy.nside2npix(nside_pixel))
        mask[self.roi.pixels] = self.mask.mask_2.mask_roi_sparse
        mag_lim_2 = mask[pix]

        mag_err_1 = self.photo_err_1(mag_lim_1 - mag_1)
        mag_err_2 = self.photo_err_2(mag_lim_2 - mag_2)
        mc_source_id = mc_source_id * numpy.ones(len(mag_1))

        # ADW: Should magnitudes be randomized by the erros?
        #mag_1 += (numpy.random.normal(size=len(mag_1)) * mag_err_1)
        #mag_2 += (numpy.random.normal(size=len(mag_2)) * mag_err_2)

        select = (mag_lim_1>mag_1)&(mag_lim_2>mag_2)

        ### # Make sure objects lie within the original cmd (should be done later...)
        ### select &= (ugali.utils.binning.take2D(self.mask.solid_angle_cmd, color, mag_1,
        ###                                       self.roi.bins_color, self.roi.bins_mag) > 0)

        logger.info("Clipping %i simulated background stars..."%(~select).sum())
        
        hdu = ugali.observation.catalog.makeHDU(self.config,mag_1[select],mag_err_1[select],
                                                mag_2[select],mag_err_2[select],
                                                lon[select],lat[select],mc_source_id[select])
        catalog = ugali.observation.catalog.Catalog(self.config, data=hdu.data)
        return catalog

    def satellite(self,stellar_mass,distance_modulus,mc_source_id=1,seed=None,**kwargs):
        """
        Create a simulated satellite. Returns a catalog object.
        """
        if seed is not None: np.random.seed(seed)

        isochrone = kwargs.pop('isochrone',self.isochrone)
        kernel    = kwargs.pop('kernel',self.kernel)

        for k,v in kwargs.items():
            if k in kernel.params.keys(): setattr(kernel,k,v)

        mag_1, mag_2 = isochrone.simulate(stellar_mass, distance_modulus)
        lon, lat     = kernel.simulate(len(mag_1))
 
        logger.info("Simulating %i satellite stars..."%len(mag_1))
        pix = ang2pix(self.config['coords']['nside_pixel'], lon, lat)

        # There is probably a better way to do this step without creating the full HEALPix map
        mask = -1. * numpy.ones(healpy.nside2npix(self.config['coords']['nside_pixel']))
        mask[self.roi.pixels] = self.mask.mask_1.mask_roi_sparse
        mag_lim_1 = mask[pix]
        mask = -1. * numpy.ones(healpy.nside2npix(self.config['coords']['nside_pixel']))
        mask[self.roi.pixels] = self.mask.mask_2.mask_roi_sparse
        mag_lim_2 = mask[pix]

        mag_err_1 = self.photo_err_1(mag_lim_1 - mag_1)
        mag_err_2 = self.photo_err_2(mag_lim_2 - mag_2)

        # Randomize magnitudes by their errors
        mag_obs_1 = mag_1+numpy.random.normal(size=len(mag_1))*mag_err_1
        mag_obs_2 = mag_2+numpy.random.normal(size=len(mag_2))*mag_err_2
        #mag_obs_1 = mag_1
        #mag_obs_2 = mag_2

        #select = numpy.logical_and(mag_obs_1 < mag_lim_1, mag_obs_2 < mag_lim_2)
        select = (mag_lim_1>mag_obs_1)&(mag_lim_2>mag_obs_2)

        # Make sure objects lie within the original cmd (should also be done later...)
        #select &= (ugali.utils.binning.take2D(self.mask.solid_angle_cmd, mag_obs_1 - mag_obs_2, mag_obs_1,self.roi.bins_color, self.roi.bins_mag) > 0)

        #return mag_1_obs[cut], mag_2_obs[cut], lon[cut], lat[cut]
        logger.info("Clipping %i simulated satellite stars..."%(~select).sum())
        mc_source_id = mc_source_id * numpy.ones(len(mag_1))
        
        hdu = ugali.observation.catalog.makeHDU(self.config,mag_obs_1[select],mag_err_1[select],
                                                mag_obs_2[select],mag_err_2[select], 
                                                lon[select],lat[select],mc_source_id[select])
        catalog = ugali.observation.catalog.Catalog(self.config, data=hdu.data)
        return catalog


    def satellite2(self,stellar_mass,distance_modulus,mc_source_id=1,seed=None,**kwargs):
        """
        Create a simulated satellite. Returns a catalog object.
        """
        if seed is not None: np.random.seed(seed)

        isochrone = kwargs.pop('isochrone',self.isochrone)
        kernel    = kwargs.pop('kernel',self.kernel)

        for k,v in kwargs.items():
            if k in kernel.params.keys(): setattr(kernel,k,v)

        mag_1, mag_2 = isochrone.simulate(stellar_mass, distance_modulus)
        lon, lat     = kernel.simulate(len(mag_1))

        logger.info("Simulating %i satellite stars..."%len(mag_1))
        pix = ang2pix(self.config['coords']['nside_pixel'], lon, lat)

        # There is probably a better way to do this step without creating the full HEALPix map
        mask = -1. * numpy.ones(healpy.nside2npix(self.config['coords']['nside_pixel']))
        mask[self.roi.pixels] = self.mask.mask_1.mask_roi_sparse
        mag_lim_1 = mask[pix]
        mask = -1. * numpy.ones(healpy.nside2npix(self.config['coords']['nside_pixel']))
        mask[self.roi.pixels] = self.mask.mask_2.mask_roi_sparse
        mag_lim_2 = mask[pix]

        mag_err_1 = self.mask.photo_err_1(mag_lim_1 - mag_1)
        mag_err_2 = self.mask.photo_err_2(mag_lim_2 - mag_2)

        # Completeness is a function of true magnitude
        method = 'step'
        if method is None or method == 'none':
            comp = np.ones(len(mag_1))
        elif self.config['catalog']['band_1_detection']:
            comp=self.mask.completeness(mag_lim_1-mag_1, method=method)
        elif not self.config['catalog']['band_1_detection']:
            comp=self.mask.completeness(mag_lim_2-mag_2, method=method)
        else:
            comp_1 = self.mask.completeness(mag_lim_1-mag_1, method=method)
            comp_2 = self.mask.completeness(mag_lim_2-mag_2, method=method)
            comp = comp_1*comp_2
        accept = comp > 1 - np.random.uniform(size=len(mag_1))

        # Randomize magnitudes by their errors
        mag_obs_1 = mag_1 + (numpy.random.normal(size=len(mag_1))*mag_err_1)
        mag_obs_2 = mag_2 + (numpy.random.normal(size=len(mag_2))*mag_err_2)

        #select = numpy.logical_and(mag_obs_1 < mag_lim_1, mag_obs_2 < mag_lim_2)
        select = (mag_lim_1>mag_obs_1)&(mag_lim_2>mag_obs_2)&accept

        ### # Make sure objects lie within the original cmd (should also be done later...)
        ### select &= (ugali.utils.binning.take2D(self.mask.solid_angle_cmd, color, mag_1,
        ###                                       self.roi.bins_color, self.roi.bins_mag) > 0)

        #return mag_1_obs[cut], mag_2_obs[cut], lon[cut], lat[cut]
        logger.info("Clipping %i simulated satellite stars..."%(~select).sum())
        mc_source_id = mc_source_id * numpy.ones(len(mag_1))
        
        hdu = ugali.observation.catalog.makeHDU(self.config,mag_obs_1[select],mag_err_1[select],
                                                mag_obs_2[select],mag_err_2[select], 
                                                lon[select],lat[select],mc_source_id[select])
        catalog = ugali.observation.catalog.Catalog(self.config, data=hdu.data)
        return catalog


    def simulate(self, seed=None, **kwargs):
        if seed is not None: np.random.seed(seed)

        logger.info("Simulating object catalog...")
        catalogs = []
        #catalogs.append(self.toy_background(seed=seed))
        catalogs.append(self.background(seed=seed))
        catalogs.append(self.satellite(seed=seed,**kwargs))
        logger.info("Merging simulated catalogs...")
        catalog = ugali.observation.catalog.mergeCatalogs(catalogs)
        nsig = (catalog.mc_source_id == 1).sum()
        nbkg = (catalog.mc_source_id == 2).sum()
        logger.info("Simulated catalog contains: %i background stars"%nbkg)
        logger.info("Simulated catalog contains: %i satellite stars"%nsig)
        return catalog

    def makeHDU(self, mag_1, mag_err_1, mag_2, mag_err_2, lon, lat, mc_source_id):
        """
        Create a catalog fits file object based on input data.

        ADW: This should be combined with the write_membership function of loglike.
        """

        if self.config['catalog']['coordsys'].lower() == 'cel' \
           and self.config['coords']['coordsys'].lower() == 'gal':
            lon, lat = ugali.utils.projector.gal2cel(lon, lat)
        elif self.config['catalog']['coordsys'].lower() == 'gal' \
           and self.config['coords']['coordsys'].lower() == 'cel':
            lon, lat = ugali.utils.projector.cel2gal(lon, lat)

        columns = [
            pyfits.Column(name=self.config['catalog']['objid_field'],
                          format = 'D',array = np.arange(len(lon))),
            pyfits.Column(name=self.config['catalog']['lon_field'],
                          format = 'D',array = lon),
            pyfits.Column(name = self.config['catalog']['lat_field'],          
                          format = 'D',array = lat), 
            pyfits.Column(name = self.config['catalog']['mag_1_field'],        
                          format = 'E',array = mag_1),
            pyfits.Column(name = self.config['catalog']['mag_err_1_field'],    
                          format = 'E',array = mag_err_1),
            pyfits.Column(name = self.config['catalog']['mag_2_field'],        
                          format = 'E',array = mag_2),
            pyfits.Column(name = self.config['catalog']['mag_err_2_field'],    
                          format = 'E',array = mag_err_2),
            pyfits.Column(name = self.config['catalog']['mc_source_id_field'], 
                          format = 'I',array = mc_source_id),
        ]

        hdu = pyfits.new_table(columns)
        return hdu


    def write(self, outfile):
        """
        """
        pass

############################################################

def satellite(isochrone, kernel, stellar_mass, distance_modulus,**kwargs):
    """
    Wrapping the isochrone and kernel simulate functions.
    """
    mag_1, mag_2 = isochrone.simulate(stellar_mass, distance_modulus)
    lon, lat     = kernel.simulate(len(mag_1))

    return mag_1, mag_2, lon, lat

############################################################


if __name__ == "__main__":
    import ugali.utils.parser
    description = "Script for executing the likelihood scan."
    parser = ugali.utils.parser.Parser(description=description)
    parser.add_config()
    parser.add_argument('outfile',metavar='outfile.fits',help='Output fits file.')
    parser.add_debug()
    parser.add_verbose()
    parser.add_seed()

    opts = parser.parse_args()
    config = Config(opts.config)
    generator = Generator(config,opts.seed)
    sim,like1,like2 = generator.run(opts.outfile)
    



#!/usr/bin/env python
import subprocess, subprocess as sub
import getpass
from collections import OrderedDict as odict
from itertools import chain
import copy

from ugali.utils.logger import logger

CLUSTERS = odict([
    ('local',['local']),
    ('lsf',['lsf','slac','kipac']),
    ('slurm',['slurm','midway','kicp']),
    ('condor',['condor','fnal']),
])

QUEUES = odict([
    ('local',[]),
    ('lsf',['express','short','medium','long','xlong','xxl','kipac-ibq','bulletmpi']),
    ('slurm',[]),
    ('condor',['local','vanilla','universe','grid']),
])

RUNLIMITS = odict([            #Hard limits
        (None,'4:00'),         # Default value
        ('express','0:01'),    # 0:01
        ('short','0:30'),      # 0:30
        ('medium','1:00'),     # 4:00
        ('long','4:00'),       # 32:00
        ('xlong','72:00'),     # 72:00
        ('xxl','168:00'),
        ('kipac-ibq','24:00'),
        ('bulletmpi','48:00'),
        ])

def factory(queue,**kwargs):
    if queue is None: queue = 'local'

    name = queue.lower()
    if name in list(chain(*QUEUES.values())):
        kwargs.setdefault('q',name)

    if name in CLUSTERS['local']+QUEUES['local']:
        batch = Local(**kwargs)
    elif name in CLUSTERS['lsf']+QUEUES['lsf']:
        batch = LSF(**kwargs)
    elif name in CLUSTERS['slurm']+QUEUES['slurm']:
        batch = Slurm(**kwargs)
    elif name in CLUSTERS['condor']+QUEUES['condor']:
        # Need to learn how to use condor first...
        batch = Condor(**kwargs)
        raise Exception("FNAL cluster not implemented")
    else:
        raise TypeError('Unexpected queue name: %s'%name)

    return batch

batchFactory = factory

class Batch(object):
    # Default options for batch submission
    _defaults = odict([])
    # Map between generic and batch specific names
    _mapping = odict([])

    def __init__(self, **kwargs):
        self.username = getpass.getuser()
        self.default_opts = copy.deepcopy(self._defaults)
        self.default_opts.update(**kwargs)
        self.submit_cmd = "submit %(opts)s %(command)s"
        self.jobs_cmd = "jobs"

    def jobs(self):
        out = self.popen(self.jobs_cmd)
        stdout = out.communicate()[0]
        return stdout

    def njobs(self):
        # Remove header line
        jobs = self.jobs()
        return len(jobs.strip().split('\n'))-1 if jobs else 0

    def popen(self, command):
        return sub.Popen(command,shell=True,
                         stdin=sub.PIPE,stdout=sub.PIPE,stderr=sub.PIPE)

    def call(self, command):
        logger.debug(command)
        return sub.call(command,shell=True)

    def remap_options(self,opts):
        for k in self._mapping.keys():
            v = opts.pop(k,None)
            if v is not None:
                opts[self._mapping[k]] = v

    def batch(self, command, jobname=None, logfile=None, **opts):
        if jobname: opts.update(jobname=jobname)
        if logfile: opts.update(logfile=logfile)
        self.remap_options(opts)
        params = dict(opts=self.parse_options(**opts),command=command)
        return self.submit_cmd%params

    def submit(self, command, jobname=None, logfile=None, **opts):
        cmd = self.batch(command, jobname, logfile, **opts)
        self.call(cmd)
        return cmd

class Local(Batch):
    def __init__(self,**kwargs):
        super(Local,self).__init__(**kwargs)
        self.jobs_cmd = "echo 0"
        self.submit_cmd = "%(command)s %(opts)s"

    def parse_options(self,**opts):
        if opts.get('logfile'): return ' | tee %(logfile)s'%opts
        return ''

class LSF(Batch):
    _defaults = odict([
        ('R','"scratch > 1 && rhel60"'),
        ('C', 0),
        #('q', 'long'),
    ])

    _mapping = odict([
        ('jobname','J'),
        ('logfile','oo')
    ])

    def __init__(self,**kwargs):
        super(LSF,self).__init__(**kwargs)

        self.jobs_cmd = "bjobs -u %s"%self.username
        self.submit_cmd = "bsub %(opts)s %(command)s"

    def runlimit(self, queue=None):
        """
        Translate queue to wallclock runlimit.
        """
        return RUNLIMITS[queue]
        
    q2w = runlimit

    def parse_options(self, **opts):
        options = odict(self.default_opts)
        options.update(opts)
        if 'n' in options.keys():
            options['a'] = 'mpirun'
            options['R'] += ' -R "span[ptile=4]"'
        options.setdefault('W',self.runlimit(options.get('q')))
        return ''.join('-%s %s '%(k,v) for k,v in options.items())
        
class Slurm(Batch):
    _defaults = odict([
        ('account','kicp'),
        ('partition','kicp-ht'),
        ('mem',10000)
    ])

    def __init__(self, **kwargs):
        super(Slurm,self).__init__(**kwargs)
        logger.warning('Slurm cluster is untested')

        self.jobs_cmd = "squeue -u %s"%self.username
        self.submit_cmd = "sbatch %(opts)s %(command)s"

    def parse_options(self, **opts):
        options = odict(self.default_opts)
        options.update(opts)
        return ''.join('--%s %s '%(k,v) for k,v in options.items())

class Condor(Batch):
    """ Not implemented yet... """
    def __init__(self):
        super(Condor,self).__init__(**kwargs)
        logger.warning('Condor cluster is untested')
        
        self.jobs_cmd = 'condor_q -u %s'%self.username
        self.submit_cmd = "csub %(opts)s %(command)s"

if __name__ == "__main__":
    import argparse
    description = "python script"
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('args',nargs=argparse.REMAINDER)
    args = parser.parse_args()




#!/usr/bin/env python
"""
A Model object is just a container for a set of Parameter.
Implements __getattr__ and __setattr__.

The model has a set of default parameters stored in Model._params. 
Careful, if these are changed, they will be changed for all 
subsequent instances of the Model.

The parameters for a given instance of a model are stored in the
Model.params attribute. This attribute is a deepcopy of 
Model._params created during instantiation.

"""
from collections import OrderedDict as odict
import numpy as np
import copy
import yaml

def indent(string,width=0): 
    return '{0:>{1}}{2}'.format('',width,string)

def asscalar(a):
    """ https://github.com/numpy/numpy/issues/4701 """
    # Do we want to check that the value is numeric?
    #if   isinstance(value, (int, long, float)): return value
    try:
        return np.asscalar(a)
    except AttributeError, e:
        return np.asscalar(np.asarray(a))


class Model(object):
    # The _params member is an ordered dictionary
    # of Parameter objects. (Should be immutable tuple.)
    _params = odict([])
    # The _mapping is an alternative name mapping
    # for the parameters in _params
    _mapping = odict([])

    def __init__(self,*args,**kwargs):
        self.name = self.__class__.__name__
        self.params = copy.deepcopy(self._params)
        self.set_attributes(**kwargs)
        #pars = dict(**kwargs)
        #for name, value in pars.items():
        #    # Raise AttributeError if attribute not found
        #    self.__getattr__(name) 
        #    # Set attribute
        #    self.__setattr__(name,value)
        
        # In case no properties were set, cache anyway
        self._cache()

    def __getattr__(self,name):
        # Return 'value' of parameters
        # __getattr__ tries the usual places first.
        #if name in self._mapping:
        #    return self.__getattr__(self._mapping[name])
        #if name in self._params:
        #    return self.getp(name).value
        #else:
        #    # Raises AttributeError
        #    return object.__getattribute__(self,name)
        if name in self._params or name in self._mapping:
            return self.getp(name).value
        else:
            # Raises AttributeError
            return object.__getattribute__(self,name)

    def __setattr__(self, name, value):
        ## Call 'set_value' on parameters
        ## __setattr__ tries the usual places first.
        #if name in self._mapping.keys():
        #    return self.__setattr__(self._mapping[name],value)
        #if name in self._params:
        #    self.setp(name, value)
        #else:
        #    return object.__setattr__(self, name, value)
        if name in self._params or name in self._mapping:
            self.setp(name, value)
        else:
            # Why is this a return statement
            return object.__setattr__(self, name, value)

    def __str__(self,indent=0):
        ret = '{0:>{2}}{1}'.format('',self.name,indent)
        if len(self.params)==0:
            pass
        else:            
            ret += '\n{0:>{2}}{1}'.format('','Parameters:',indent+2)
            width = len(max(self.params.keys(),key=len))
            for name,value in self.params.items():
                par = '{0!s:{width}} : {1!r}'.format(name,value,width=width)
                ret += '\n{0:>{2}}{1}'.format('',par,indent+4)
        return ret

    def getp(self, name):
        """ 
        Get the named parameter.

        Parameters
        ----------
        name : string
            The parameter name.

        Returns
        -------
        param : 
            The parameter object.
        """
        name = self._mapping.get(name,name)
        return self.params[name]

    def setp(self, name, value=None, bounds=None, free=None, errors=None):
        """ 
        Set the value (and bounds) of the named parameter.

        Parameters
        ----------
        name : string
            The parameter name.
        value: 
            The value of the parameter
        bounds: None
            The bounds on the parameter
        Returns
        -------
        None
        """
        name = self._mapping.get(name,name)
        self.params[name].set(value,bounds,free,errors)
        self._cache(name)

    def set_attributes(self, **kwargs):
        """
        Set a group of attributes (parameters and members).  Calls
        `setp` directly, so kwargs can include more than just the
        parameter value (e.g., bounds, free, etc.).
        """
        kwargs = dict(kwargs)
        for name,value in kwargs.items():
            # Raise AttributeError if param not found
            self.__getattr__(name) 
            # Set attributes
            try: self.setp(name,**value)
            except TypeError:
                try:  self.setp(name,*value)
                except (TypeError,KeyError):  
                    self.__setattr__(name,value)
        
    def todict(self):
        ret = odict(name = self.__class__.__name__)
        ret.update(self.params)
        return ret

    def dump(self):
        return yaml.dump(self.todict())

    def _cache(self, name=None):
        """ 
        Method called in _setp to cache any computationally
        intensive properties after updating the parameters.

        Parameters
        ----------
        name : string
           The parameter name.

        Returns
        -------
        None
        """
        pass

    #@property
    #def params(self):
    #    return self._params


class Parameter(object):
    """
    Parameter class for storing a value, bounds, and freedom.

    Adapted from `MutableNum` from https://gist.github.com/jheiv/6656349
    """
    __value__ = None
    __bounds__ = None
    __free__ = False
    __errors__ = None

    def __init__(self, value, bounds=None, free=None, errors=None): 
        self.set(value,bounds,free,errors)

    # Comparison Methods
    def __eq__(self, x):        return self.__value__ == x
    def __ne__(self, x):        return self.__value__ != x
    def __lt__(self, x):        return self.__value__ <  x
    def __gt__(self, x):        return self.__value__ >  x
    def __le__(self, x):        return self.__value__ <= x
    def __ge__(self, x):        return self.__value__ >= x
    def __cmp__(self, x):       return 0 if self.__value__ == x else 1 if self.__value__ > 0 else -1
    # Unary Ops
    def __pos__(self):          return +self.__value__
    def __neg__(self):          return -self.__value__
    def __abs__(self):          return abs(self.__value__)
    # Bitwise Unary Ops
    def __invert__(self):       return ~self.__value__
    # Arithmetic Binary Ops
    def __add__(self, x):       return self.__value__ + x
    def __sub__(self, x):       return self.__value__ - x
    def __mul__(self, x):       return self.__value__ * x
    def __div__(self, x):       return self.__value__ / x
    def __mod__(self, x):       return self.__value__ % x
    def __pow__(self, x):       return self.__value__ ** x
    def __floordiv__(self, x):  return self.__value__ // x
    def __divmod__(self, x):    return divmod(self.__value__, x)
    def __truediv__(self, x):   return self.__value__.__truediv__(x)
    # Reflected Arithmetic Binary Ops
    def __radd__(self, x):      return x + self.__value__
    def __rsub__(self, x):      return x - self.__value__
    def __rmul__(self, x):      return x * self.__value__
    def __rdiv__(self, x):      return x / self.__value__
    def __rmod__(self, x):      return x % self.__value__
    def __rpow__(self, x):      return x ** self.__value__
    def __rfloordiv__(self, x): return x // self.__value__
    def __rdivmod__(self, x):   return divmod(x, self.__value__)
    def __rtruediv__(self, x):  return x.__truediv__(self.__value__)
    # Bitwise Binary Ops
    def __and__(self, x):       return self.__value__ & x
    def __or__(self, x):        return self.__value__ | x
    def __xor__(self, x):       return self.__value__ ^ x
    def __lshift__(self, x):    return self.__value__ << x
    def __rshift__(self, x):    return self.__value__ >> x
    # Reflected Bitwise Binary Ops
    def __rand__(self, x):      return x & self.__value__
    def __ror__(self, x):       return x | self.__value__
    def __rxor__(self, x):      return x ^ self.__value__
    def __rlshift__(self, x):   return x << self.__value__
    def __rrshift__(self, x):   return x >> self.__value__
    # ADW: Don't allow compound assignments
    ## Compound Assignment
    #def __iadd__(self, x):      self.set(self + x); return self
    #def __isub__(self, x):      self.set(self - x); return self
    #def __imul__(self, x):      self.set(self * x); return self
    #def __idiv__(self, x):      self.set(self / x); return self
    #def __imod__(self, x):      self.set(self % x); return self
    #def __ipow__(self, x):      self.set(self **x); return self
    # Casts
    def __nonzero__(self):      return self.__value__ != 0
    def __int__(self):          return self.__value__.__int__()    
    def __float__(self):        return self.__value__.__float__()  
    def __long__(self):         return self.__value__.__long__()   
    # Conversions
    def __oct__(self):          return self.__value__.__oct__()    
    def __hex__(self):          return self.__value__.__hex__()    
    def __str__(self):          return self.__value__.__str__()    
    # Random Ops
    def __index__(self):        return self.__value__.__index__()  
    def __trunc__(self):        return self.__value__.__trunc__()  
    def __coerce__(self, x):    return self.__value__.__coerce__(x)
    # Represenation
    # ADW: This should probably be __str__ not __repr__
    def __repr__(self):         
        if self.bounds:
            # if bounds is None this breaks
            return "%s(%s, [%s, %s], %s)"%(self.__class__.__name__, self.value,self.bounds[0],self.bounds[1],self.free)
        else:
            return "%s(%s, [%s, %s], %s)"%(self.__class__.__name__, self.value,self.bounds,self.bounds,self.free)

    # Return the type of the inner value
    def innertype(self):  return type(self.__value__)

    @property
    def bounds(self):
        return self.__bounds__

    @property
    def value(self):
        return self.__value__

    @property
    def free(self):
        return self.__free__

    @property
    def errors(self):
        return self.__errors__

    def item(self): 
        """ For asscalar """
        return self.value

    def check_bounds(self, value):
        if self.__bounds__ is None:
            return
        if not (self.__bounds__[0] <= value <= self.__bounds__[1]):
            msg="Value outside bounds: %.2g [%.2g,%.2g]"
            msg=msg%(value,self.__bounds__[0],self.__bounds__[1])
            raise ValueError(msg)

    def set_bounds(self, bounds):
        if bounds is None: return
        self.__bounds__ = [asscalar(b) for b in bounds]

    def set_value(self, value):
        if value is None: return
        self.check_bounds(value)
        self.__value__ = asscalar(value)

    def set_free(self, free):
        if free is None: return
        else: self.__free__ = bool(free)

    def set_errors(self, errors):
        if errors is None: return
        self.__errors__ = [asscalar(e) for e in errors]

    def set(self, value=None, bounds=None, free=None, errors=None):
        # Probably want to reset bounds if set fails
        self.set_bounds(bounds)
        self.set_value(value)
        self.set_free(free)
        self.set_errors(errors)

    def todict(self):
        return odict(value=self.value,bounds=self.bounds,
                     free=self.free,errors=self.errors)

    def dump(self):
        return yaml.dump(self)

    @staticmethod
    def representer(dumper, data):
        """ 
        http://stackoverflow.com/a/14001707/4075339
        http://stackoverflow.com/a/21912744/4075339 
        """
        tag = yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG
        return dumper.represent_mapping(tag,data.todict().items(),flow_style=True)

def odict_representer(dumper, data):
    """ http://stackoverflow.com/a/21912744/4075339 """
    # Probably belongs in a util
    return dumper.represent_mapping(
        yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,data.items())

yaml.add_representer(odict,odict_representer)
yaml.add_representer(Parameter,Parameter.representer)

if __name__ == "__main__":
    import argparse
    description = "python script"
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('args',nargs=argparse.REMAINDER)
    opts = parser.parse_args(); args = opts.args



#!/usr/bin/env python
"""
toolkit for working with healpix
"""
import numpy
import numpy as np
import healpy

############################################################

def superpixel(subpix, nside_subpix, nside_superpix):
    """
    Return the indices of the super-pixels which contain each of the sub-pixels.
    """
    if nside_subpix==nside_superpix: return subpix
    theta, phi =  healpy.pix2ang(nside_subpix, subpix)
    return healpy.ang2pix(nside_superpix, theta, phi)

def subpixel(superpix, nside_superpix, nside_subpix):
    """
    Return the indices of sub-pixels (resolution nside_subpix) within the super-pixel with (resolution nside_superpix).
    """
    if nside_superpix==nside_subpix: return superpix
    vec = healpy.pix2vec(nside_superpix, superpix)
    radius = numpy.degrees(2. * healpy.max_pixrad(nside_superpix))
    subpix = query_disc(nside_subpix, vec, radius)
    pix_for_subpix = superpixel(subpix,nside_subpix,nside_superpix)
    # Might be able to speed up array indexing...
    return subpix[pix_for_subpix == superpix]

############################################################

def phi2lon(phi): return np.degrees(phi)
def lon2phi(lon): return np.radians(lon)

def theta2lat(theta): return 90. - np.degrees(theta)
def lat2theta(lat): return np.radians(90. - lat)

def pix2ang(nside, pix):
    """
    Return (lon, lat) in degrees instead of (theta, phi) in radians
    """
    theta, phi =  healpy.pix2ang(nside, pix)
    lon = phi2lon(phi)
    lat = theta2lat(theta)
    return lon, lat

def ang2pix(nside, lon, lat, coord='GAL'):
    """
    Input (lon, lat) in degrees instead of (theta, phi) in radians
    """
    theta = np.radians(90. - lat)
    phi = np.radians(lon)
    return healpy.ang2pix(nside, theta, phi)

def ang2vec(lon, lat):
    theta = lat2theta(lat)
    phi = lon2phi(lon)
    vec = healpy.ang2vec(theta, phi)
    return vec

pixToAng = pix2ang
angToPix = ang2pix
angToVec = ang2vec

############################################################

def healpixMap(nside, lon, lat, fill_value=0.):
    """
    Input (lon, lat) in degrees instead of (theta, phi) in radians.
    Returns HEALPix map at the desired resolution 
    """
    pix = angToPix(nside, lon, lat)
    m = numpy.histogram(pix, numpy.arange(healpy.nside2npix(nside) + 1))[0].astype(float)
    if fill_value != 0.:
        m[m == 0.] = fill_value
    return m

############################################################

def in_pixels(lon,lat,pixels,nside):
    """
    Check if (lon,lat) in pixel list.
    """
    pix = ang2pix(nside,lon,lat)
    return numpy.in1d(pix,pixels)

def index_pixels(lon,lat,pixels,nside):
   """
   Find the index for object amoung a subset of healpix pixels.
   Set index of objects outside the pixel subset to -1

   # ADW: Not really safe to set index = -1 (accesses last entry); 
   # -np.inf would be better, but breaks other code...
   """
   pix = ang2pix(nside,lon,lat)
   # pixels should be pre-sorted, otherwise...???
   index = numpy.searchsorted(pixels,pix)
   if numpy.isscalar(index):
       if not numpy.in1d(pix,pixels).any(): index = -1
   else:
       # Find objects that are outside the roi
       #index[numpy.take(pixels,index,mode='clip')!=pix] = -1
       index[~numpy.in1d(pix,pixels)] = -1
   return index

############################################################

def query_disc(nside, vec, radius, inclusive=False, fact=4, nest=False):
    """
    Wrapper around healpy.query_disc to deal with old healpy implementation.

    nside : int
      The nside of the Healpix map.
    vec : float, sequence of 3 elements
      The coordinates of unit vector defining the disk center.
    radius : float
      The radius (in degrees) of the disk
    inclusive : bool, optional
      If False, return the exact set of pixels whose pixel centers lie 
      within the disk; if True, return all pixels that overlap with the disk,
      and maybe a few more. Default: False
    fact : int, optional
      Only used when inclusive=True. The overlapping test will be done at
      the resolution fact*nside. For NESTED ordering, fact must be a power of 2,
      else it can be any positive integer. Default: 4.
    nest: bool, optional
      if True, assume NESTED pixel ordering, otherwise, RING pixel ordering

    """
    try: 
        # New-style call (healpy 1.6.3)
        return healpy.query_disc(nside, vec, numpy.radians(radius), inclusive, fact, nest)
    except: 
        # Old-style call (healpy 0.10.2)
        return healpy.query_disc(nside, vec, numpy.radians(radius), nest, deg=False)

def ang2disc(nside, lon, lat, radius, inclusive=False, fact=4, nest=False):
    """
    Wrap `query_disc` to use lon, lat, and radius in degrees.
    """
    vec = ang2vec(lon,lat)
    return query_disc(nside,vec,radius,inclusive,fact,nest)

angToDisc = ang2disc

def get_interp_val(m, lon, lat, *args, **kwargs):
    return healpy.get_interp_val(m, lat2theta(lat), lon2phi(lon), *args, **kwargs)

############################################################

if __name__ == "__main__":
    import argparse
    description = "python script"
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument('args',nargs=argparse.REMAINDER)
    opts = parser.parse_args(); args = opts.args



"""
Classes to handle initial mass functions (IMFs).
"""

import numpy

############################################################

class IMF:

    def __init__(self, type='chabrier'):
        """
        Initialize an instance of an initial mass function.
        """

        self.type = type

        if self.type == 'chabrier':
            self.pdf = chabrierIMF
        else:
            print 'WARNING: initial mass function type %s not recognized'%(self.type)

    def integrate(self, mass_min, mass_max, log_mode=True, weight=False, steps=10000):
        """
        Numerically integrate initial mass function.

        INPUTS:
            mass_min: minimum mass bound for integration (solar masses)
            mass_max: maximum mass bound for integration (solar masses)
            log_mode[True]: use logarithmic steps in stellar mass as oppose to linear
            weight[False]: weight the integral by stellar mass
            steps: number of numerical integration steps
        OUTPUT:
            result of integral
        """
        if log_mode:
            d_log_mass = (numpy.log10(mass_max) - numpy.log10(mass_min)) / float(steps)
            log_mass = numpy.linspace(numpy.log10(mass_min), numpy.log10(mass_max), steps)
            mass = 10.**log_mass

            if weight:
                return numpy.sum(mass * d_log_mass * self.pdf(mass, log_mode=True))
            else:
                return numpy.sum(d_log_mass * self.pdf(mass, log_mode=True))
        else:
            d_mass = (mass_max - mass_min) / float(steps)
            mass = numpy.linspace(mass_min, mass_max, steps)

            if weight:
                return numpy.sum(mass * d_mass * self.pdf(mass, log_mode=False))
            else:
                return numpy.sum(d_mass * self.pdf(mass, log_mode=False))

############################################################

def chabrierIMF(mass, log_mode=True, a=1.31357499301):
    """
    Chabrier initial mass function. Put the reference for the formula here.
    
    INPUTS:
        mass: stellar mass (solar masses)
        log_mode[True]: return number per logarithmic mass range, i.e., dN/dlog(M)
        a[1.31357499301]: normalization; normalized by default to the mass interval 0.1--100 solar masses
    OUTPUTS:
        number per (linear or logarithmic) mass range, i.e., dN/dM or dN/dlog(M) where mass unit is solar masses
    """
    log_mass = numpy.log10(mass)
    b = 0.279087531047 # Where did this hard-coded number come from??
    if log_mode:
        # Number per logarithmic mass range, i.e., dN/dlog(M)
        return ((log_mass <= 0.) * a * numpy.exp(-1. * (log_mass - numpy.log10(0.079))**2 / (2 * (0.69**2)))) + \
               ((log_mass > 0.) * a * b * mass**(-1.3))
    else:
        # Number per linear mass range, i.e., dN/dM
        return (((log_mass <= 0.) * a * numpy.exp(-1. * (log_mass - numpy.log10(0.079))**2 / (2 * (0.69**2)))) + \
                ((log_mass > 0.) * a * b * mass**(-1.3))) / (mass * numpy.log(10))

############################################################



#!/usr/bin/env python

class Prior(object):
    def __call__(self, value):
        return self.value(value)
 
    def value(self, value):
        msg = "`value` must be implmented in child class"
        raise Exception(msg)

class UniformPrior(Prior): 
    def value(self, value):
        return 1.0

class InversePrior(Prior):
    def value(self, value):
        return 1.0/value

class BetaPrior(Prior):
    def value(self, value):
        return scipy.stats.beta.pdf(value,0.5,0.5)

if __name__ == "__main__":
    import argparse
    description = "python script"
    parser = argparse.ArgumentParser(description=description)
    opts = parser.parse_args()

